// This is the Core of quimup.
// It handles the communication between the sever and the user-accessible windows.

#include <qpopupmenu.h>
#include <qprocess.h>
#include <qmessagebox.h> // error msg etc
#include <time.h> // mpd uptime
#include <qapplication.h> // to get the default font


int pl_x, pl_y, tr_x, tr_y, tr_w, tr_h;
bool b_trFirstShow;
bool b_trMustPopUp;

void qm_Core::init()
{
    this->hide(); // Core has no interface
    hangon = 0;
    curPlaylist = -999;
    curSong = -1;
    conn = NULL;
    b_dbaseUpdating = FALSE;
    b_commandBusy = FALSE;
    b_statCheckBusy = FALSE;
    b_PLupdating = FALSE;
    b_trMustPopUp = FALSE;
    refreshInt = 300;
    coreSrvStatus = MPD_STATUS_STATE_UNKNOWN;
    lastDBupdate = 0;

    // Get all the saved settings - to be applied here, in init.
    loadSettings();

    // window style to be used
    WFlags flags = (WStyle_DialogBorder |WStyle_SysMenu | WStyle_Customize) & ~WStyle_ContextHelp;

    // Create the main (player) window
    plWindow = new qm_Player(this,0,0,flags);
    connect(plWindow, SIGNAL(crToggleRepeat()), this, SLOT(crToggleRepeat()));
    connect(plWindow, SIGNAL(crToggleRandom()), this, SLOT(crToggleRandom()));
    connect(plWindow, SIGNAL(crToggleTracks()), this, SLOT(crToggleTracks()));
    connect(plWindow, SIGNAL(crToggleOptions()), this, SLOT(crToggleOptions()));
    connect(plWindow, SIGNAL(crSendVolume(int)), this, SLOT(crSendVolume(int)));
    connect(plWindow, SIGNAL(crSendProgress(double)), this, SLOT(crSendProgress(double)));
    connect(plWindow, SIGNAL(crSendNext()), this, SLOT(crSendNext()));
    connect(plWindow, SIGNAL(crSendPrev()), this, SLOT(crSendPrev()));
    connect(plWindow, SIGNAL(crSendPlay()), this, SLOT(crSendPlay()));
    connect(plWindow, SIGNAL(crSendStop()), this, SLOT(crSendStop()));
    connect(plWindow, SIGNAL(crTrayClicked(int)), this, SLOT(crTrayClicked(int)));
    connect(plWindow, SIGNAL(crPlayerClosed()), this, SLOT(crPlayerClosed()));
    connect(plWindow, SIGNAL(crStoreTimeLeft(bool)), this, SLOT(crStoreTimeLeft(bool)));
    connect(plWindow, SIGNAL(crSetTrayTip(const QString &)), this, SLOT(crSetTrayTip(const QString &)));

    //Create the options window
    opWindow = new qm_Settings(this,0,0,flags);
    connect(opWindow, SIGNAL(crGetsOptions(const qm_OptStruct &)), this, SLOT(crGetsOptions(const qm_OptStruct &)));
    connect(opWindow, SIGNAL(crServerConnect(const QString &, int, const QString &)), this, SLOT(crServerConnect(const QString &, int, const QString &)));
    // read some settings from mpdconf
    getMPDopts();


    // Create the Tracks window
    trWindow = new qm_Tracks(this,0,0,flags);
    connect(trWindow, SIGNAL(crConnectError()), this, SLOT(crConnectError()));
    connect(trWindow, SIGNAL(crPlaylistIncrement()), this, SLOT(crPlaylistIncrement()));
    connect(trWindow, SIGNAL(crCurSongChanged(int)), this, SLOT(crCurSongChanged(int)));
    connect(trWindow, SIGNAL(crUpdatePlaylist()), this, SLOT(crUpdatePlaylist()));
    connect(trWindow, SIGNAL(crGetsTrData(int, bool)), this, SLOT(crGetsTrData(int, bool)));
    connect(trWindow, SIGNAL(crDBaseUpdate()), this, SLOT(crDBaseUpdate()));
    connect(trWindow, SIGNAL(crToggleTracks()), this, SLOT(crToggleTracks()));
    connect(trWindow, SIGNAL(crSendStop()), this, SLOT(crSendStop()));
    // pass on the stored font size
    trSetListFonts(TheOps.listFsize);
    // restore the Tracks window's dataMode and history-tag setting
    if (trDataMode > 3) // not dir, artist, album or playlist
     trDataMode = 2; // artist
    trSetData(trDataMode, b_trMarkPlayed);
    // pass on playlist path and max size
    trGetsPlistInfo(mpdOptions.plistMax, mpdOptions.playlistPath, mpdOptions.musicPath);
    // restore position
    QSettings settings;
    tr_x = settings.readNumEntry( "/quimup/Pos/TracksX", 50 );
    tr_y = settings.readNumEntry( "/quimup/Pos/TracksY", 50 );
    tr_w = settings.readNumEntry( "/quimup/Pos/TracksWidth", 520 );
    tr_h = settings.readNumEntry( "/quimup/Pos/TracksHeight", 656 );
    trWindow->move(tr_x,tr_y);
    trWindow->resize(tr_w,tr_h);
    b_trFirstShow = TRUE;

    // Create the System Tray Icon
    b_usesystray = TheOps.useSystray;
    if (b_usesystray)
    {
    trayPopmenu = new QPopupMenu();

    if (TheOps.colorise)
    {
        QColor fCol = QColor(0, 10, 40);
        QPalette pal (trayPopmenu->palette());
        pal.setColor(QColorGroup::ButtonText, fCol);
        pal.setColor(QColorGroup::Text, fCol);
        pal.setColor(QColorGroup::Foreground, fCol);
        pal.setColor(QColorGroup::Background, QColor(218, 220, 224));
        pal.setColor(QColorGroup::Button, QColor(218, 220, 224));
        pal.setColor(QColorGroup::Highlight, QColor(75, 100, 125));
        trayPopmenu->setPalette(pal);
    }

    QFont trayFont = QApplication::font();
    trayFont.setPointSize(11);
    trayPopmenu->setFont(trayFont);

    trayPause = QPixmap::fromMimeSource( "st_paused.png" );
    trayPlay = QPixmap::fromMimeSource( "st_playing.png" );
    trayStop = QPixmap::fromMimeSource( "st_stopped.png" );
    trayNoCon = QPixmap::fromMimeSource( "st_noconn.png" );
    tpausIcon =  QPixmap::fromMimeSource( "st_pause.png" );
    tplayIcon  = QPixmap::fromMimeSource( "st_play.png" );
    QPixmap tprevIcon =  QPixmap::fromMimeSource( "st_prev.png" );
    QPixmap tnextIcon  = QPixmap::fromMimeSource( "st_next.png" );
    QPixmap tstopIcon  = QPixmap::fromMimeSource( "st_stop.png" );
    QPixmap texitIcon  = QPixmap::fromMimeSource( "st_exit.png" );

    trayPopmenu->setCheckable(FALSE);
    trayPopmenu->insertItem(tprevIcon, "Previous", this, SLOT(crSendPrev()));
    trayPopmenu->insertItem(tstopIcon, "Stop", this, SLOT(crSendStop()));
    trayPopmenu->insertItem(tplayIcon, "Play", this, SLOT(crSendPlay()));
    trayPopmenu->insertItem(tnextIcon, "Next", this, SLOT(crSendNext()));
    trayPopmenu->insertItem(texitIcon, "Quit Quimup", this, SLOT(quimupKill()));

    connect(trayPopmenu, SIGNAL(aboutToShow()), this, SLOT(toggleTrayPlay()));
    QString theTip = "Quimup";
    quimupTray = new TrayIcon(trayNoCon, theTip, trayPopmenu, this, "quimupTray");
    connect(quimupTray, SIGNAL(sendClicked(int)), this, SLOT(crTrayClicked(int)) );
   }

    mpdStatusPoller = new QTimer(this);
    // Checks the status of the server every 'refreshInt' mseconds.
    // Loop is started in setConnStatus, when connection is established.
    connect(mpdStatusPoller, SIGNAL(timeout()), this, SLOT(statusCheck()));

    reconnectLoop = new QTimer(this);
    // Tries to reconnect every second.
    // Loop is started in various places when (conn == NULL) is observed.
    connect(reconnectLoop, SIGNAL(timeout()), this, SLOT(serverReconnect()));

    if (b_usesystray)
       quimupTray->show();

    if (b_timeLeft)
    plToggleTimeLeft();

    setConnStatus(FALSE);
    stsSetConMsg(FALSE, "Not Connected");

    if (TheOps.autoConnect)
    {
    crServerConnect(TheOps.sName, TheOps.sPort, TheOps.sPassword);
    // if autoconnect failed and TheOps.startServer: start MPD and try again
    if (conn == NULL && TheOps.startServer)
    {
        if (StartServer())
            crServerConnect(TheOps.sName, TheOps.sPort, TheOps.sPassword);
    }
    }

    // restore saved positions
    pl_x = settings.readNumEntry( "/quimup/Pos/PlayerX", 100 );
    pl_y = settings.readNumEntry( "/quimup/Pos/PlayerY", 100 );
    plWindow->move(pl_x,pl_y);
    // open the main window
    plWindow->show();
    // The window does not get its own pos right when it is moved
    // while not shown. So we restore the pos once more.
    plWindow->move(pl_x,pl_y);
}


// Signal from Tracks Window
void qm_Core::crGetsTrData(int mode, bool his)
{
    trDataMode = mode;
    b_trMarkPlayed = his;
}

// signal from player window
void qm_Core::crSetTrayTip( const QString &thetip )
{
    if (b_usesystray)
    quimupTray->setToolTip(thetip);
}


void qm_Core::setConnStatus(bool isConn)
{
    // disable / enable the systray pop-up commands
    if (b_usesystray)
    {
    trayPopmenu->setItemEnabled(trayPopmenu->idAt(0), isConn);
    trayPopmenu->setItemEnabled(trayPopmenu->idAt(1), isConn);
    trayPopmenu->setItemEnabled(trayPopmenu->idAt(2), isConn);
    trayPopmenu->setItemEnabled(trayPopmenu->idAt(3), isConn);
    }

    trSetStatus(isConn);
    setStatistics(isConn); // for opWindow.
    coreSrvStatus = MPD_STATUS_STATE_UNKNOWN;

    if (isConn)
    {
        printf("Connection OK\n");
    reconnectLoop->stop();
    mpdStatusPoller->start(refreshInt);
    // Reset the playlist and the current song
    curSong = -1;
    curPlaylist = -1;
    // Pass server info to trWindow (so it can connect too)
    trSetServer(serverName, serverPort, serverPassword);
    // set windows and systray
    plChangeStatus(MPD_STATUS_STATE_STOP); // will display a '*'
    stsSetConMsg(TRUE, ".."); // string will be replaced
        if (b_usesystray)
      quimupTray->setIcon(trayStop);
    }

    else // Not connected or pulling the plug
    {
    mpdStatusPoller->stop();
    b_statCheckBusy = FALSE;
    // Close all connections.
    if (conn != NULL)
    {
        mpd_closeConnection(conn);
        conn = NULL;
    }
    trCloseConn();
    // set windows and systray
    plSetErrorMsg("Disconnected."); // which calls plChangeStatus(-99);
    // nb. stsSetConMsg is called when (re)connecting.
        if (b_usesystray)
       quimupTray->setIcon(trayNoCon);
    trSetCurSong(-1, MPD_STATUS_STATE_UNKNOWN);
    }
}

void qm_Core::crServerConnect(const QString & host, int port, const QString & password)
{
    // Don't ErrorCheck() here. An error would only start the reconnectloop.

    // Save the server info (for the Tracks Window)
    // Do not store in TheOps - that is only done after OK in options window.
    serverName = host;
    serverPort = port;
    serverPassword = password;

    // no server, no go.
    if (serverName.isEmpty())
    {
        stsSetConMsg(FALSE, "No host name - try 'localhost'.");
    return;
    }
    // user port number must be > 1024
    if (port <= 1024)
    {
        stsSetConMsg(FALSE, "Not a valid port - try '6600'.");
    return;
    }
    // OK, a serious attempt.
    // setConnStatus to disconnect and stop the mpdStatusPoller.
    setConnStatus(FALSE);
    b_connecting = TRUE;

    // Connect
    conn = mpd_newConnection(serverName, port ,2);

    // If connection failed try 4 more tmes at 0.25 sec intervals (in case mpd is starting up)
    int count = 0;
    while ((conn->error || conn == NULL) && count < 4)
    {
    if (conn != NULL)
    {
            mpd_clearError(conn);
        mpd_closeConnection(conn);
        conn = NULL;
    }
    // wait 250 msec
        usleep(250000);
    conn = mpd_newConnection(serverName, port ,2);
    count++;
    }
    if (conn->error || conn == NULL) // Still no luck
    {
    if (conn != NULL)
    {
            mpd_clearError(conn);
        mpd_closeConnection(conn);
        conn = NULL;
    }
    stsSetConMsg(conn, "Status: Failed to connect.");
    b_connecting = FALSE;
    return;
    }
    else
    {
    printf("Connected to MPD (tries: " + QString::number(count+1) + ")\n");
        // Send Password, if provided.
    if (!serverPassword.isEmpty())
    {
        mpd_sendPasswordCommand(conn, serverPassword);
        mpd_finishCommand(conn);
        if (conn->error)
        {
            mpd_clearError(conn);
            mpd_closeConnection(conn);
            conn = NULL;
            stsSetConMsg(conn, "Status: Invalid password");
            plSetErrorMsg("Invalid password");
            printf("Invalid password\n");
            b_connecting = FALSE;
            return;
        }
    }
    // If password was not sent it may still be required.
    // Let's see if mpd is willing to do something, like set Repeat ...
    mpd_sendRepeatCommand(conn, b_repeat);
    mpd_finishCommand(conn);
    if (conn->error)
    {

        mpd_clearError(conn);
        mpd_closeConnection(conn);
        conn = NULL;
        stsSetConMsg(conn, "Status: Password required?");
        printf("No response. Password required?\n");
        plSetErrorMsg("Password required?");
        b_connecting = FALSE;
        return;
    }
    else
    // Worked! - finish setting repeat
    plSetRepeat(b_repeat);

    // Set Crossfade
    mpd_sendCrossfadeCommand(conn, TheOps.xfadeTime);
    mpd_finishCommand(conn);
    if (conn->error)
        printf("Xfade command error\n");


    // Set Random
        mpd_sendRandomCommand(conn, b_random);
    mpd_finishCommand(conn);
    plSetRandom(b_random);

    // Final check
    if (conn->error)
    {
        mpd_clearError(conn);
        mpd_closeConnection(conn);
        conn = NULL;
        stsSetConMsg(conn, "Status: Disconnected (command error)");
    }
    else
        setConnStatus(TRUE);
    }
    b_connecting = FALSE;
}

void qm_Core::getOutputs()
{
    // remove previous results
    (TheOps.oDevices).clear();
    // get current output devices
    if (conn != NULL)
    {
        mpd_OutputEntity * otity = NULL;
    crLockStatUpdate();
    mpd_sendOutputsCommand(conn);
        if (ErrorCheck(0, "mpd_sendOutputsCommand"))
    {
        while ((otity = mpd_getNextOutput(conn)))
        {
        (TheOps.oDevices).append(*otity);
        }
    }
    mpd_finishCommand(conn);
    crUnlockStatUpdate();
        if (otity != NULL)
        mpd_freeOutputElement(otity);
    }
}



void qm_Core::setStatistics(bool isConn)
{
    // Double check the connection! When the mpdStatusPoller encounters an error
    // setConnStatus(FALSE) is called, which calls this function, which can
    // crUnlockStatUpdate and restart the loop, which will find the same error... etc.
    if (!isConn || conn == NULL)
    {
    stsSetVersion("Is MPD running?");
    trSetStatistics(-1 , 0, 0);
    return;
    }

    crLockStatUpdate();
    QString version = "MPD ";
    version += QString::number(conn->version[0]) + "." + QString::number(conn->version[1]);
    // version += "." + ""QString::number(conn->version[2]); // third applies to the API version, not the main MPD version
    stsSetVersion(version);

    mpd_Stats * sData;
    mpd_sendStatsCommand(conn);
    sData = mpd_getStats(conn);
    mpd_finishCommand(conn);
    if (!ErrorCheck(2, "mpd_sendStatsCommand") || sData == NULL)
    {
    trSetStatistics(-1 , 0, 0);
    if (sData != NULL)
        mpd_freeStats(sData);
    crUnlockStatUpdate();
    return;
    }

    trSetStatistics(sData->numberOfArtists , sData->numberOfAlbums, sData->numberOfSongs);

    if (sData != NULL)
    mpd_freeStats(sData);

    // first trSetStatistics, above, to reset b_noSearchNow etc. in qm_Tracks.
    if (lastDBupdate != sData->dbUpdateTime)
    {
    lastDBupdate = sData->dbUpdateTime;
    // If a database update was done in Quimup, data will be reloaded here
    // only if the database has actually changed
    trReloadData(TRUE);
    }

    crUnlockStatUpdate();
}


void qm_Core::statusCheck()
{
    if (!ErrorCheck(4, "Starting statusCheck"))
    return;
    // Skip cycle if - previous has not finished - a command is busy
    if (b_commandBusy || b_statCheckBusy)
    {
    printf("Core skipped a statuscheck\n");
    return;
    }
    // Else start checking
    b_statCheckBusy = TRUE;

    // Get the current server status
    mpd_sendStatusCommand(conn);
    mpd_Status * servStatus = mpd_getStatus(conn);
    mpd_finishCommand(conn);
    if (!ErrorCheck(5, "mpd_sendStatsCommand") || servStatus == NULL)
    {
        b_statCheckBusy = FALSE;
    if (servStatus != NULL)
        mpd_freeStatus(servStatus);
        return;
    }

    // server has not found a supported mixer
    if (servStatus->volume == MPD_STATUS_NO_VOLUME)
    {
    if (!b_volDisabled)
    {
        plVolDisabled(true);
        b_volDisabled = true;
    }

    b_statCheckBusy = FALSE;
    if (servStatus != NULL)
        mpd_freeStatus(servStatus);
        return;
    }

    // volume
    if (b_volDisabled)
    {
    plVolDisabled(false);
    b_volDisabled = false;
    }
    if (curVolume != (servStatus->volume))
    {
    curVolume = (servStatus->volume);
    plSetVolume(curVolume);
    }

    // playlist
    if (curPlaylist < servStatus->playlist)
    {
    // store the new pl number
    printf("Playlist reset \n");
    curPlaylist = servStatus->playlist;
    crUpdatePlaylist();
    }

    // Random
    if (b_random != servStatus->random)
    {
    b_random = !b_random;
    plSetRandom(b_random);
    }

    // Repeat
    if (b_repeat != servStatus->repeat)
    {
    b_repeat = !b_repeat;
    plSetRepeat(b_repeat);
    }

    // is the dbase (still) updating?
    bool b_dbaseUpisDone = !servStatus->updatingDb;

    bool restarted = FALSE;
    // Let's see what's happened to the song
    if (curSong == servStatus->song) // song did not change
    {
    // restarted playing
    if (servStatus->state == MPD_STATUS_STATE_PLAY && coreSrvStatus != MPD_STATUS_STATE_PLAY)
    {
        restarted = TRUE;
    }
    else
    // paused
    if (servStatus->state == MPD_STATUS_STATE_PAUSE && coreSrvStatus != MPD_STATUS_STATE_PAUSE)
    {
            if (b_usesystray)
          quimupTray->setIcon(trayPause);
        plChangeStatus(MPD_STATUS_STATE_PAUSE);
        // 'pause' the playlist
        trSetCurSong(curSong, MPD_STATUS_STATE_PAUSE);
    }
    else
    // stopped
    if (servStatus->state == MPD_STATUS_STATE_STOP && coreSrvStatus != MPD_STATUS_STATE_STOP)
    {
            if (b_usesystray)
          quimupTray->setIcon(trayStop);
        plChangeStatus(MPD_STATUS_STATE_STOP);
        // 'stop' the playlist
        trSetCurSong(curSong, MPD_STATUS_STATE_STOP);
    }
    }

     // the song changed, or crCurSongChanged was called, or song was restarted
    if(curSong != servStatus->song || restarted)
    {
    // get the new song.
    mpd_InfoEntity * ntity = NULL;
    mpd_sendCurrentSongCommand(conn);
    ntity = mpd_getNextInfoEntity(conn);
    mpd_finishCommand(conn);
     // no song (like after last in the list has finished)
    if (ntity == NULL || !ErrorCheck(8,"mpd_getNextInfoEntity"))
    {
        // Don't repeat this
        curSong = servStatus->song;
        // clean up
        if (ntity != NULL)
            mpd_freeInfoEntity(ntity);
        if (servStatus != NULL)
            mpd_freeStatus(servStatus);
        // can't call crSendStop so do it here:
        mpd_sendStopCommand(conn);
            mpd_finishCommand(conn);
        ErrorCheck(8,"mpd_sendStopCommand");
        b_statCheckBusy = FALSE;
            // in case DB update has finished
            if (b_dbaseUpdating && b_dbaseUpisDone)
            {
            setStatistics(TRUE);
            b_dbaseUpdating = FALSE;
            }
        // update playlist, player and tray-icon
            if (b_usesystray)
                quimupTray->setIcon(trayStop);
            coreSrvStatus = MPD_STATUS_STATE_STOP;
            plChangeStatus(coreSrvStatus);
            trSetCurSong(curSong, -1); // curSong is 0, -1 will set 'wait' icon
        plNewSong(NULL, -1);
        return;
    }

    // mpdsong is used when fonts are changed
    theSong = mpd_songDup (ntity->info.song);

    // release memory
    if (ntity != NULL)
        mpd_freeInfoEntity(ntity);

    // Store current song nr.
    curSong = servStatus->song;

    songTotalTime = servStatus->totalTime;

    // update playlist, player and tray-icon
    if (servStatus->state == MPD_STATUS_STATE_PLAY)
    {
        trSetCurSong(curSong, MPD_STATUS_STATE_PLAY);
        plChangeStatus(MPD_STATUS_STATE_PLAY);
        plNewSong(theSong, curSong);
        plUpdate(servStatus->elapsedTime, servStatus->bitRate, servStatus->sampleRate);
            if (b_usesystray)
            quimupTray->setIcon(trayPlay);
    }
    else
    if (servStatus->state == MPD_STATUS_STATE_PAUSE) // can happen at start up
    {
        trSetCurSong(curSong, MPD_STATUS_STATE_PAUSE);
        plChangeStatus(MPD_STATUS_STATE_PAUSE);
        plNewSong(theSong, curSong);
        plUpdate(servStatus->elapsedTime, servStatus->bitRate, servStatus->sampleRate);
            if (b_usesystray)
            quimupTray->setIcon(trayPause);
    }
    else
    {
        trSetCurSong(curSong, MPD_STATUS_STATE_STOP);
        plNewSong(theSong, curSong);
        plChangeStatus(MPD_STATUS_STATE_STOP);
            if (b_usesystray)
            quimupTray->setIcon(trayStop);
    }
    }

    // update track info while playing
    if (servStatus->state == MPD_STATUS_STATE_PLAY)
    {
    songTotalTime = servStatus->totalTime; // required for the progress bar
    plUpdate(servStatus->elapsedTime, servStatus->bitRate, servStatus->sampleRate);
    }

    // Store the current status
    coreSrvStatus = servStatus->state;
    if (servStatus != NULL)
        mpd_freeStatus(servStatus);

    ErrorCheck(9,"Closing StatusCheck");
    b_statCheckBusy = FALSE;

    // DBase update was requested and has finished
    // Don't do this above: setStatistics will call crLockStatUpdate
    // which will wait forever for b_statCheckBusy to finish.
    if (b_dbaseUpdating && b_dbaseUpisDone)
    {
    setStatistics(TRUE);
    b_dbaseUpdating = FALSE;
    }
}


void qm_Core::crSendStop()
{
    if (conn == NULL)
    return;
    crLockStatUpdate();
    mpd_sendStopCommand(conn);
    mpd_finishCommand(conn);
    ErrorCheck(10, "mpd_sendStopCommand");
    crUnlockStatUpdate();
 }


void qm_Core::crSendPlay()
{
    if (conn == NULL)
    return;
    crLockStatUpdate();
    if (coreSrvStatus == MPD_STATUS_STATE_PLAY)
    {
    // If playing, send pause
    mpd_sendPauseCommand(conn, 1);
        mpd_finishCommand(conn);
    ErrorCheck(11, "mpd_sendPauseCommand 1");
    }
    else
    if (coreSrvStatus == MPD_STATUS_STATE_PAUSE)
    {
    // If paused, send continue
    mpd_sendPauseCommand(conn, 0);
        mpd_finishCommand(conn);
    ErrorCheck(11, "mpd_sendPauseCommand 0");
    }
    else
    if (coreSrvStatus == MPD_STATUS_STATE_STOP)
    {
    // If stopped, play from the top
    mpd_sendPlayCommand(conn, MPD_PLAY_AT_BEGINNING);
        mpd_finishCommand(conn);
    ErrorCheck(11, "mpd_sendPlayCommand 0");
    }
    crUnlockStatUpdate();
}


void qm_Core::crSendPrev()
{
    if (conn == NULL || coreSrvStatus == MPD_STATUS_STATE_STOP)
    return;

    crLockStatUpdate();
    mpd_sendPrevCommand(conn);
    mpd_finishCommand(conn);
    ErrorCheck(12, "mpd_sendPrevCommand");
    crUnlockStatUpdate();
}


void qm_Core::crSendNext()
{
    if (conn == NULL || coreSrvStatus == MPD_STATUS_STATE_STOP)
    return;

    crLockStatUpdate();
    mpd_sendNextCommand(conn);
    mpd_finishCommand(conn);
    ErrorCheck(14, "mpd_sendNextCommand");
    crUnlockStatUpdate();
}


void qm_Core::crDBaseUpdate()
{
    if (conn == NULL)
    return;

    crLockStatUpdate();
    mpd_sendUpdateCommand(conn, (char*)"");
    mpd_finishCommand(conn);
    ErrorCheck(15,"mpd_sendUpdateCommand");
    b_dbaseUpdating = TRUE;
    crUnlockStatUpdate();
    // mpdStatusPoller does stuff when the update has finished
}

void qm_Core::crSendProgress(double fraction)
{
    if (conn == NULL || coreSrvStatus != MPD_STATUS_STATE_PLAY)
    return;
    crLockStatUpdate();
    mpd_sendSeekCommand(conn, curSong, (int)(fraction * songTotalTime));
    mpd_finishCommand(conn);
    ErrorCheck(16,"mpd_sendSeekCommand");
    crUnlockStatUpdate();
}


void qm_Core::crSendVolume(int volume)
{
    if (conn == NULL) return;
    crLockStatUpdate();
    mpd_sendSetvolCommand(conn, volume);
    mpd_finishCommand(conn);
    ErrorCheck(17,"mpd_sendSetvolCommand");
    crUnlockStatUpdate();
}

bool qm_Core::ErrorCheck(int nr, QString msg)
{
    // no connection - try to reconnect
    if (conn == NULL)
    {
    setConnStatus(FALSE);
    stsSetConMsg(conn, "Status: Reconnecting.");
    plSetErrorMsg("Lost connection...");
    reconnectLoop->start(1000);
        printf("Core Error: " + QString::number(nr) + " (No Connection)\n");
    return FALSE;
    }

    // If an error occurred
    if (conn->error)
    {
    // If it is a non-fatal error (no permission etc.).
    if (conn->error == MPD_ERROR_ACK)
    {
        plSetErrorMsg("MPD ack error");
        QString sError = QString::fromUtf8(conn->errorStr);
            printf("Core Ack Error: " + QString::number(nr) + " " + msg + "\n" + sError + "\n");
        mpd_clearError(conn);
        return TRUE;
    }

    // A fatal error occurred: disconnnect.
    mpd_clearError(conn);
    setConnStatus(FALSE);
    stsSetConMsg(conn, "Status: Reconnecting.");
    plSetErrorMsg("Lost connection...");
    reconnectLoop->start(1000);
        printf("Core Fatal Error: " + QString::number(nr) + " " + msg + "\n");
    return FALSE;
    }

    // No errors, apparently
    mpd_clearError(conn);
    return TRUE;
}


// called by the statuscheck when the playlist has changed
void qm_Core::crUpdatePlaylist()
{
    while (b_PLupdating) {}; // wait if busy
    b_PLupdating = TRUE;

    // variables
    qm_SongPtrList TheList;
    //mpd_Song aSong;
    mpd_Connection * playlistConn;
    mpd_InfoEntity * ntity;

    // open a new connection
    playlistConn = mpd_newConnection (serverName, serverPort, 10);

    if (!PupErrorCheck(playlistConn))
    {
     b_PLupdating = FALSE;
     return;
    }

    if (!serverPassword.isEmpty())
    {
        mpd_sendPasswordCommand(playlistConn, serverPassword);
    mpd_finishCommand(playlistConn);
    if (!PupErrorCheck(playlistConn))
    {

        b_PLupdating = FALSE;
        printf("Playlist: mpd password error\n");
        return;
    }
    }

    // use -1 to get the WHOLE list
    mpd_sendPlaylistInfoCommand(playlistConn, -1);
    int i = 0;
    if (PupErrorCheck(playlistConn))
    {
    while((ntity = mpd_getNextInfoEntity(playlistConn)))
    {
            if (!PupErrorCheck(playlistConn))
        {
             b_PLupdating = FALSE;
             return;
        }
            if (ntity->type != MPD_INFO_ENTITY_TYPE_SONG)
            {
                mpd_freeInfoEntity(ntity);
                continue;
            }

            mpd_Song *theSong = mpd_songDup (ntity->info.song);
            TheList.append(theSong);
            mpd_freeInfoEntity(ntity);

        i++;
        }
    }
    mpd_finishCommand(playlistConn); // sendPlaylistInfoCommand
    if (!PupErrorCheck(playlistConn))
    printf("Error getting playlist from mpd\n");

    mpd_closeConnection(playlistConn);
    playlistConn = NULL;

    trSetServerPlaylist(TheList);

    // if the list is empty, also clear the player
    if (i == 0)
    {
    plNewSong(NULL, -1);
    plChangeStatus(MPD_STATUS_STATE_STOP);
    }
    curSong = -1;
    b_PLupdating = FALSE;
}


// ErrorCheck for the Playlist Update
bool qm_Core::PupErrorCheck(mpd_Connection *& plconn)
{
    if (plconn == NULL)
    {
        return FALSE;
    }
    else
    if (plconn->error)
    {
    printf("Core playlist update error\n");
    mpd_clearError(plconn);
    mpd_closeConnection(plconn);
    plconn = NULL;
    return FALSE;
    }
    else
    return TRUE;
}


//  Gracefully close the program
void qm_Core::closeEvent( QCloseEvent * a)
{
    printf("Closing down\n");
    saveSettings();
    // stop the mpdStatusPoller
    mpdStatusPoller->stop();
    // Close connections
    trCloseConn();
    if (conn)
    mpd_closeConnection(conn);
    conn = NULL;
    // execute exit command
    if (TheOps.killServer)
    KillServer();
    // and to play it safe...
    if (b_usesystray)
    {
    if (quimupTray)
            delete quimupTray;
    if (trayPopmenu)
            delete trayPopmenu;
    }
    // Close windows
    trReallyClose();
    if (trWindow)
    delete trWindow;
    plReallyClose();
    if (plWindow)
    delete plWindow;
    opWindow->close();
    if (opWindow)
    delete opWindow;
    // Shutdown and accept the CloseEvent
    Shutdown();
    a->accept();
    printf("Goodbye!\n");
}


void qm_Core::crToggleOptions()
{
    if (opWindow->isShown())
    opWindow->close();
    else
    {
    // send status - string is replaced when conn is TRUE.
    stsSetConMsg(conn, "Disconnected");
    // send settings
        getOutputs(); // first!
        stsGetsMPData(mpdOptions);
    stsGetsOptions(TheOps);
    opWindow->show();
    }
}

void qm_Core::crToggleTracks()
{
    // note that a close-event is diverted to pass through here
    if (trWindow->isShown())
    {
        // The window does not get its own pos right when it
        // is not shown. So we store x,y,w and h first.
        tr_x = trWindow->x();
        tr_y = trWindow->y();
        tr_w = trWindow->width();
        tr_h = trWindow->height();
    trWindow->hide();
    }
    else
    {
    trWindow->show();
    if (b_trFirstShow)
    {
        trWindow->move(tr_x,tr_y);
        trWindow->resize(tr_w,tr_h);
        // The column widths must be set after show():
            QSettings settings;
        int dbasePanelWidth = settings.readNumEntry( "/quimup/General/dbasePanelW", 200 );
        int plistPanelWidth = settings.readNumEntry( "/quimup/General/plistPanelW", 200 );
        trSetSplitter(dbasePanelWidth, plistPanelWidth);
        b_trFirstShow = FALSE;
    }
    }
}

void qm_Core::crTrayClicked(int mbutton)
{
    // -99 is a fake click from plWindow
    if (mbutton == LeftButton || mbutton == -99)
    {
    if (plWindow->isShown())
        {
            // Store x and y pos
        pl_x = plWindow->x();
        pl_y = plWindow->y();
            plWindow->hide();

        }
    else
        plWindow->show();

    if (trWindow->isShown())
    {
        b_trMustPopUp = TRUE;
        trWindow->hide();
    }
    else
        if (b_trMustPopUp)
        {
        trWindow->show();
        b_trMustPopUp = FALSE;
        }
    }
    else
    if (mbutton == MidButton)
        crSendPlay();
}

// sets the pause/play icon upon opening the menu
void qm_Core::toggleTrayPlay()
{
    if (!b_usesystray)
    return;

    if (coreSrvStatus == MPD_STATUS_STATE_PLAY)
    trayPopmenu->changeItem(trayPopmenu->idAt(2), tpausIcon, "Pause");
    else
    trayPopmenu->changeItem(trayPopmenu->idAt(2), tplayIcon, "Play");
}

// called by the tracks window. Prevents an update in the mpdStatusPoller.
void qm_Core::crPlaylistIncrement()
{
    curPlaylist++;
}


// trWindow reports a lost connection
void qm_Core::crConnectError()
{
    setConnStatus(FALSE);
    stsSetConMsg(conn, "Status: Reconnecting.");
    plSetErrorMsg("Lost connection...");
    reconnectLoop->start(1000);
}


// Called by the reconnect loop
void qm_Core::serverReconnect()
{
    // Don't ErrorCheck() here!
    // An error here would start the reconnectloop, which would find the error here...

    // skip cycle if previous has not finished
    if (b_connecting)
    return;

    b_connecting = TRUE;
    // close existing erroneous connection
    if (conn != NULL)
    {
        if (conn->error)
            mpd_clearError(conn);
    mpd_closeConnection(conn);
    conn = NULL;
    }
    // no server, no go.
    if (serverName.isEmpty())
    {
    reconnectLoop->stop();
        b_connecting = FALSE;
    return;
    }
    // try to connect
    conn = mpd_newConnection(serverName, serverPort,1);
    if (conn->error || conn == NULL)
    {
    if (conn != NULL)
    {
                mpd_clearError(conn);
        mpd_closeConnection(conn);
        conn = NULL;
    }
    b_connecting = FALSE;
    return;
    }

    // login if there is a password
    if (!serverPassword.isEmpty())
    {
    mpd_sendPasswordCommand(conn, serverPassword);
    }
    // and check what the result is
    if (conn->error)
    {
        mpd_clearError(conn);
    // close erroneous connection
    mpd_closeConnection(conn);
    conn = NULL;
    b_connecting = FALSE;
    return;
    }
    // Connected!
    setConnStatus(TRUE); // this will stop the loop
    b_connecting = FALSE;
}


// mpdStatusPoller is locked before command execution.
void qm_Core::crLockStatUpdate()
{
    while (b_commandBusy)
    {
    // Wait for previous command to finish
    }

    while(b_statCheckBusy)
    {
    // Wait for statuscheck to finish
    }

    // and (b)lock the mpdStatusPoller
    b_commandBusy = TRUE;
}


// mpdStatusPoller is unlocked after command execution.
void qm_Core::crUnlockStatUpdate()
{
    b_statCheckBusy = FALSE;
    b_commandBusy = FALSE;
}


void qm_Core::crToggleRandom()
{
    if (conn == NULL) return;
    crLockStatUpdate();
    mpd_sendRandomCommand(conn, !b_random);
    mpd_finishCommand(conn);
    ErrorCheck(18, "mpd_sendRandomCommand");
    crUnlockStatUpdate();
    b_random = !b_random;
    plSetRandom(b_random);
}


void qm_Core::crToggleRepeat()
{
    if (conn == NULL) return;
    crLockStatUpdate();
    mpd_sendRepeatCommand(conn, !b_repeat);
    mpd_finishCommand(conn);
    ErrorCheck(19, "mpd_sendRepeatCommand");
    crUnlockStatUpdate();
    b_repeat = !b_repeat;
    plSetRepeat(b_repeat);
}


void qm_Core::crGetsOptions(const qm_OptStruct & newOps)
{
    // update crossfade time
    if (TheOps.xfadeTime != newOps.xfadeTime)
    {
    if (mpdStatusPoller->isActive() && conn != NULL)
    {
        crLockStatUpdate();
        mpd_sendCrossfadeCommand(conn, newOps.xfadeTime);
        mpd_finishCommand(conn);
        ErrorCheck(20, "mpd_sendCrossfadeCommand");
        crUnlockStatUpdate();
    }
    }

    // output settings
    OutDevList::iterator cit;
    OutDevList theList = newOps.oDevices;
    for ( cit = theList.begin(); cit != theList.end(); ++cit )
    {
    mpd_OutputEntity curDev = *cit;
    int ID = curDev.id;
    int enbld = curDev.enabled;
    if (conn != NULL)
    {
        crLockStatUpdate();
        if (enbld)
                mpd_sendEnableOutputCommand(conn, ID);
        else
                mpd_sendDisableOutputCommand(conn, ID);
        mpd_finishCommand(conn);
        ErrorCheck(21, "mpd_sendDisableOutputCommand");
            crUnlockStatUpdate();
    }
    }

    // adjust player window
    if (TheOps.titleFsize != newOps.titleFsize)
    {
    plRedrawTitle(newOps.titleFsize);
        if (coreSrvStatus == MPD_STATUS_STATE_PLAY || coreSrvStatus == MPD_STATUS_STATE_PAUSE)
       plNewSong(theSong, curSong);
    }
    if (TheOps.albumFsize != newOps.albumFsize)
    plRedrawAlbum(newOps.albumFsize);
    if (TheOps.infoFsize != newOps.infoFsize)
    plRedrawInfo(newOps.infoFsize);
    // adjust fonts in tracks window
    if (TheOps.listFsize != newOps.listFsize)
    trSetListFonts(newOps.listFsize);

    // finally transfer all options
    TheOps = newOps;
    // oDevices must not go round and round...
    (TheOps.oDevices).clear();
    // and save - in case the program crashes later :)
    saveSettings();
}


void qm_Core::saveSettings()
{
    QSettings settings;
    // Window Settings
    if (plWindow->isShown())
    {
        pl_x = plWindow->x();
        pl_y = plWindow->y();
    }
    settings.writeEntry( "/quimup/Pos/PlayerX", pl_x );
    settings.writeEntry( "/quimup/Pos/PlayerY", pl_y );
    if (trWindow->isShown())
    {
        tr_x = trWindow->x();
        tr_y = trWindow->y();
        tr_w = trWindow->width();
        tr_h = trWindow->height();
    }
    settings.writeEntry( "/quimup/Pos/TracksX", tr_x );
    settings.writeEntry( "/quimup/Pos/TracksY", tr_y );
    settings.writeEntry( "/quimup/Pos/TracksWidth", tr_w );
    settings.writeEntry( "/quimup/Pos/TracksHeight", tr_h );
    // general Settings
    settings.writeEntry( "/quimup/General/TimeLeft", b_timeLeft );
    settings.writeEntry( "/quimup/General/Random", b_random );
    settings.writeEntry( "/quimup/General/Repeat", b_repeat );
    settings.writeEntry( "/quimup/General/UseSysTray", TheOps.useSystray );
    settings.writeEntry( "/quimup/General/colorise", TheOps.colorise );
    settings.writeEntry( "/quimup/General/TitleFontSize", TheOps.titleFsize );
    settings.writeEntry( "/quimup/General/AlbumFontSize", TheOps.albumFsize );
    settings.writeEntry( "/quimup/General/ListFontSize", TheOps.listFsize );
    settings.writeEntry( "/quimup/General/InfoFontSize", TheOps.infoFsize );
    settings.writeEntry( "/quimup/General/DataMode", trDataMode );
    settings.writeEntry( "/quimup/General/ShowHistory", b_trMarkPlayed );
    settings.writeEntry( "/quimup/General/albumArt", TheOps.albumArt );
    settings.writeEntry( "/quimup/General/listFont", TheOps.listFont );
    // Server Settings
    settings.writeEntry( "/quimup/Server/StartServer", TheOps.startServer );
    settings.writeEntry( "/quimup/Server/StartCommand", TheOps.startCmd );
    settings.writeEntry( "/quimup/Server/KillServer", TheOps.killServer );
    settings.writeEntry( "/quimup/Server/KillCommand", TheOps.killCmd );
    settings.writeEntry( "/quimup/Server/AutoConnect", TheOps.autoConnect );
    settings.writeEntry( "/quimup/Server/CrossFadeTime", TheOps.xfadeTime );
    settings.writeEntry( "/quimup/Server/ServerName", TheOps.sName );
    settings.writeEntry( "/quimup/Server/ServerPort", TheOps.sPort );
    if (TheOps.sPassword != "***")
        settings.writeEntry( "/quimup/Server/ServerPassword", TheOps.sPassword );
    settings.writeEntry( "/quimup/Server/configPath", TheOps.configPath );
}


// signal from plWindow
void qm_Core::crStoreTimeLeft(bool tLeft)
{
    b_timeLeft = tLeft;
}

// Called from init
void qm_Core::loadSettings()
{
    QSettings settings;
    // Server Settings
    TheOps.startCmd = settings.readEntry( "/quimup/Server/StartCommand", "mpd" );
    TheOps.startServer = settings.readBoolEntry( "/quimup/Server/StartServer", TRUE );
    TheOps.killCmd = settings.readEntry( "/quimup/Server/KillCommand", "mpd --kill" );
    TheOps.killServer = settings.readBoolEntry( "/quimup/Server/KillServer", FALSE );
    TheOps.autoConnect = settings.readBoolEntry( "/quimup/Server/AutoConnect", TRUE );
    TheOps.xfadeTime = settings.readNumEntry( "/quimup/Server/CrossFadeTime", 0 );
    TheOps.sName = settings.readEntry( "/quimup/Server/ServerName", "localhost" );
    TheOps.sPassword = settings.readEntry( "/quimup/Server/ServerPassword", "" );
    TheOps.sPort = settings.readNumEntry( "/quimup/Server/ServerPort", 6600 );
    TheOps.configPath = settings.readEntry( "/quimup/Server/configPath", "x" );
    // check the config path
    if (TheOps.configPath == "x" || !(QFile::exists (TheOps.configPath)))
    {
    QString inHomedir = QDir::homeDirPath();
    inHomedir.append("/.mpdconf");
    if (QFile::exists (inHomedir))
    {
        TheOps.configPath = inHomedir;
    }
    else
    {
        if (QFile::exists ("/etc/mpd.conf"))
           TheOps.configPath = "/etc/mpd.conf";
        else
           TheOps.configPath = "";
        }
        // save this NOW so the player can use it.
        settings.writeEntry( "/quimup/Server/configPath", TheOps.configPath );
    }
    // General Settings
    TheOps.albumArt = settings.readEntry( "/quimup/General/albumArt", "" );
    TheOps.listFont = settings.readEntry( "/quimup/General/listFont", "SansSerif" );
    TheOps.titleFsize = settings.readNumEntry( "/quimup/General/TitleFontSize", 10 );
    TheOps.infoFsize = settings.readNumEntry( "/quimup/General/InfoFontSize", 9 );
    TheOps.listFsize = settings.readNumEntry( "/quimup/General/ListFontSize", 10 );
    TheOps.albumFsize = settings.readNumEntry( "/quimup/General/AlbumFontSize", 8 );
    TheOps.useSystray = settings.readBoolEntry( "/quimup/General/UseSysTray", TRUE );
    TheOps.colorise = settings.readBoolEntry( "/quimup/General/colorise", TRUE );
    trDataMode = settings.readNumEntry( "/quimup/General/DataMode", 0 );
    b_trMarkPlayed = settings.readBoolEntry("/quimup/General/ShowHistory", TRUE );
    b_timeLeft = settings.readBoolEntry( "/quimup/General/TimeLeft", FALSE );
    // These are sent to MPD in crServerConnect
    b_random = settings.readBoolEntry( "/quimup/General/Random", FALSE );
    b_repeat = settings.readBoolEntry( "/quimup/General/Repeat", FALSE );
}

// Start the MPD server
// Called from loadsettings, if MPD is not running && TheOps.startServer && autoconnect
bool qm_Core::StartServer()
{
    QStringList Cmd_and_Pars( QStringList::split( " ", TheOps.startCmd ));
    QProcess mpdProc(Cmd_and_Pars);

    if (!mpdProc.start())
    {
        plSetErrorMsg("Failed to start MPD");
        printf("Failed to start MPD\n");
        return FALSE;
    }
    printf("Started MPD\n");
    return TRUE;
}

// Kill the MPD server
// Called from CloseEvent, if TheOps.killServer
void qm_Core::KillServer()
{
    QStringList Cmd_and_Pars( QStringList::split( " ", TheOps.killCmd ));
    QProcess mpdProc(Cmd_and_Pars);
    mpdProc.start();
}


void qm_Core::crPlayerClosed()
{
    if ( !b_usesystray )
    quimupKill();
}


// opens the mpd config file and reads some settings
void qm_Core::getMPDopts()
{
    QFile conf;
    bool InfoFound = TRUE;
    // defaults:
    mpdOptions.playlistPath = "default";
    mpdOptions.musicPath = "default";
    mpdOptions.audiOutput = "default";
    mpdOptions.mixerControl = "not defined";
    mpdOptions.user = "not defined";
    mpdOptions.replayGain = "not defined";
    mpdOptions.chars = "not defined";
    mpdOptions.metaData = "default (no comments)";
    // configPath was checked when TheOps were loaded
    mpdOptions.configPath = TheOps.configPath;

    // get the mpd config file.
    if (mpdOptions.configPath == "")
    {
    QMessageBox::information( this,
    "Quimup", "Could not locate mpd.conf or .mpdconf.\n"
    "Please set the path in Settings.\n");
    InfoFound = FALSE;
    }
    else
    conf.setName(mpdOptions.configPath);


    if (InfoFound)
    {
      if(conf.open( IO_ReadOnly ))
      {
    QTextStream stream( &conf );
    QString line;
    int i;

    while (!stream.atEnd() )
    {
        line = stream.readLine().simplifyWhiteSpace();
        if (line.startsWith("playlist_dir", FALSE))
        {
            // find the first "
            i = line.find('"') + 1;
            // keep everything to the right of it
            line = line.right(line.length() - i);
            // remove the closing " (and more)
            i = line.find('"');
            line.truncate(i);
            // if last char is not a slash, add it
            if (line[line.length() - 1] != '/')
            line = line + "/";
            mpdOptions.playlistPath = line;
        }
        if (line.startsWith("music_dir", FALSE))
        {
            i = line.find('"') + 1;
            line = line.right(line.length() - i);
            i = line.find('"');
            line.truncate(i);
            if (line[line.length() - 1] != '/')
                line = line + "/";
            mpdOptions.musicPath = line;
        }
        if (line.startsWith("type", FALSE) || line.startsWith("ao_driver", FALSE))
        {
            i = line.find('"') + 1;
            line = line.right(line.length() - i);
            i = line.find('"');
            line.truncate(i);
            if (mpdOptions.audiOutput == "default")
              mpdOptions.audiOutput = line;
            else
              (mpdOptions.audiOutput).append(", "+line);
        }
        if (line.startsWith("mixer_control", FALSE))
        {
            i = line.find('"') + 1;
            line = line.right(line.length() - i);
            i = line.find('"');
            line.truncate(i);
            mpdOptions.mixerControl = line;
        }
        if (line.startsWith("user", FALSE))
        {
            i = line.find('"') + 1;
            line = line.right(line.length() - i);
            i = line.find('"');
            line.truncate(i);
            mpdOptions.user = line;
        }
        if (line.startsWith("replaygain", FALSE) && !line.startsWith("replaygain_pre", FALSE))
        {
            i = line.find('"') + 1;
            line = line.right(line.length() - i);
            i = line.find('"');
            line.truncate(i);
            mpdOptions.replayGain = line;
        }
        if (line.startsWith("filesystem_charset", FALSE))
        {
            i = line.find('"') + 1;
            line = line.right(line.length() - i);
            i = line.find('"');
            line.truncate(i);
            mpdOptions.chars = line;
        }
        if (line.startsWith("metadata_to_use", FALSE))
        {
            i = line.find('"') + 1;
            line = line.right(line.length() - i);
            i = line.find('"');
            line.truncate(i);
            line.remove(' ');
            line.replace(',', ", ");
            mpdOptions.metaData = line;
        }
        if (line.startsWith("max_playlist", FALSE))
        {
            i = line.find('"') + 1;
            line = line.right(line.length() - i);
            i = line.find('"');
            line.truncate(i);
            mpdOptions.plistMax = line.toInt(0, 10);
            if (mpdOptions.plistMax < 0)
                mpdOptions.plistMax = 0;
            //mpdOptions.playlistMax = line;
        }
    }
    conf.close();
      }
      else
      {
    InfoFound = FALSE;
    QMessageBox::information( this,
    "Quimup", "Could not open configuration file.\n ("
        + mpdOptions.configPath + ")\nDo you have access rights?\n");
    InfoFound = FALSE;
      }
    }

    mpdOptions.infoFound = InfoFound;
}


void qm_Core::quimupKill()
{
    this->close();
}

// signal from Tracks window
void qm_Core::crCurSongChanged(int csong)
{
    curSong = csong;
}

