#include "qm_ListItem.h"

// destructor
qm_ListItem::~qm_ListItem()
{
    if(theSong) delete theSong;
    if(theDir) delete theDir;
    if(theList) delete theList;
    b3_drawDropLine = 0;
    itemStatus = 0;
    itemType = 0;
}

////// DATABASEVIEW ///////

// Header items, like "Search results:"
qm_ListItem::qm_ListItem(QListView *parent, QString label, int iStatus)
    : QListViewItem(parent, label)
{
    theSong =  0;
    theDir = 0;
    theList = 0;
    b3_drawDropLine = 0;
    itemStatus = iStatus;
    itemType = -1;
    b3_wasPlayed = 0;
}


// Directory
qm_ListItem::qm_ListItem ( QListViewItem *parent, mpd_Directory & aDir, QString label)
    : QListViewItem(parent, label)
{
    theDir =  new mpd_Directory(aDir);
    theSong = 0;
    theList = 0;
    b3_drawDropLine = 0;
    itemStatus = LITEM_STAT_NORMAL;
    itemType = -1;
    b3_wasPlayed = 0;
}

// Playlist
qm_ListItem::qm_ListItem ( QListViewItem *parent, mpd_PlaylistFile & aList, QString label)
    : QListViewItem(parent, label)
{
    theList =  new mpd_PlaylistFile(aList);
    theSong = 0;
    theDir = 0;
    b3_drawDropLine = 0;
    itemStatus = LITEM_STAT_NORMAL;
    itemType = -1;
    b3_wasPlayed = 0;
}

// Artist, Album, Message
qm_ListItem::qm_ListItem(QListViewItem *parent, QString label, int iStatus)
    : QListViewItem(parent, label)
{
    theSong =  0;
    theDir = 0;
    theList = 0;
    b3_drawDropLine = 0;
    itemStatus = iStatus;
    itemType = -1;
    b3_wasPlayed = 0;
}

// Playlist sub-item (and some more)
qm_ListItem::qm_ListItem( QListViewItem *parent, QListViewItem *after, QString label, int iStatus)
	: QListViewItem(parent, after, label)
{
	theSong =  0;
	theDir = 0;
	theList = 0;
	b3_drawDropLine = 0;
	itemStatus = iStatus;
	itemType = -1;
    	b3_wasPlayed = 0;
}


// Song
qm_ListItem::qm_ListItem( QListViewItem *parent, mpd_Song & aSong, QString label)
    : QListViewItem(parent, label)
{
    theSong =  new mpd_Song(aSong);
    theDir = 0;
    theList = 0;
    b3_drawDropLine = 0;
    itemStatus = LITEM_STAT_NORMAL;
    itemType = -1;
    b3_wasPlayed = 0;
}

/////// PLAYLISTVIEW ////////

// first item in the playlistview
qm_ListItem::qm_ListItem( QListView *parent, mpd_Song & aSong, QPixmap pxm, QString label1, QString label2, QString label3 , QString label4)
    : QListViewItem(parent, label1, label2, label3, label4)
{
    theSong =  new mpd_Song(aSong);
    theDir = 0;
    theList = 0;
    b3_drawDropLine = 0;
    itemStatus = LITEM_STAT_NORMAL;
    itemType = -1;
    b3_wasPlayed = 0;
    setPixmap(1, pxm);
}


// items after the first one in the playlistview
qm_ListItem::qm_ListItem ( QListView *parent, QListViewItem *after, mpd_Song & aSong, QPixmap pxm, QString label1, QString label2, QString label3, QString label4)
    : QListViewItem(parent, after, label1, label2, label3, label4)
{
    theSong =  new mpd_Song(aSong);
    theDir = 0;
    theList = 0;
    b3_drawDropLine = 0;
    itemStatus = LITEM_STAT_NORMAL;
    itemType = -1;
    b3_wasPlayed = 0;
    setPixmap(1, pxm);
}

//////////////////////////////


mpd_Song qm_ListItem::getSong()
{
    return *theSong;
}


mpd_Directory qm_ListItem::getDir()
{
    return *theDir;
}


mpd_PlaylistFile qm_ListItem::getList()
{
    return *theList;
}

void qm_ListItem::setWasPlayed(char hide_no_yes) // -1, 0, 1
{
	b3_wasPlayed = hide_no_yes;
}

char qm_ListItem::getWasPlayed()
{
	return b3_wasPlayed;
}

void qm_ListItem::paintCell( QPainter *p, const QColorGroup & cg, int column, int width, int align )
{
     QColorGroup grp(cg);
     switch (itemStatus)
     {
	case LITEM_STAT_CURRENT:
	{
		grp.setColor(QColorGroup::Text, QColor(0, 28, 132));
		grp.setColor(QColorGroup::Base, QColor(233, 239, 247));
		QListViewItem::paintCell(p, grp, column, width, align);
		break;
	}
	case LITEM_STAT_PAUSED:
	{
		grp.setColor(QColorGroup::Text, QColor(0, 28, 132));
		grp.setColor(QColorGroup::Base, QColor(233, 239, 247));
		QListViewItem::paintCell(p, grp, column, width, align);
		break;
	}
	case LITEM_STAT_STOPPED:
	{
		grp.setColor(QColorGroup::Base, QColor(233, 239, 247));
		QListViewItem::paintCell(p, grp, column, width, align);
		break;
	}
	case LITEM_STAT_MESSAGE:
	{
		grp.setColor(QColorGroup::Text, QColor(128, 128, 128));
		QListViewItem::paintCell(p, grp, column, width, align);
		break;
	}
	case LITEM_STAT_PLITEM:
	{
		grp.setColor(QColorGroup::Text, QColor(65, 85, 105));
		QListViewItem::paintCell(p, grp, column, width, align);
		break;
	}
	case LITEM_STAT_HEADER:
	{
		grp.setColor(QColorGroup::Text, QColor(65, 85, 105));
		grp.setColor(QColorGroup::Base, QColor(218, 226, 235));
		QListViewItem::paintCell(p, grp, column, width, align);
        	p->setPen( QColor(140, 152, 160) );
		int h = (*this).height() - 1;
        	p->drawLine(0, h ,width, h);
		break;
	}
	default:
	{
		if (b3_wasPlayed == 1) // mark 'played'
		{
			grp.setColor(QColorGroup::Text, QColor(70, 90, 125));
		        QListViewItem::paintCell(p, grp, column, width, align);
		}
		else // change nothing and paint
		QListViewItem::paintCell(p, cg, column, width, align);
	}
    }

    if(b3_drawDropLine != 0)
    {
        p->setPen( QColor(0, 0, 128) );
        if (b3_drawDropLine < 0) // -1 is 'AT the BOTTOM'
	{
		int h = (*this).height() - 1;
        	p->drawLine(0, h ,width, h);
	}
	else
		p->drawLine(0,0,width,0); // 1 is 'AT the TOP'
    }
}


void qm_ListItem::setDrawDropLine(char  bottom_no_top)
{
    b3_drawDropLine = bottom_no_top; // -1, 0, 1
}


char qm_ListItem::getDrawDropLine()
{
    return b3_drawDropLine;
}


void qm_ListItem::setItemStatus(int  iStatus)
{
    itemStatus = iStatus;
}


int qm_ListItem::getItemStatus()
{
    return itemStatus;
}


void qm_ListItem::setType(int type)
{
    itemType = type;
}


int qm_ListItem::getType()
{
    return itemType;
}

void qm_ListItem::setAlbum(const QString & album)
{
    theAlbum = album;
}

void qm_ListItem::setArtist(const QString & artist)
{
    theArtist = artist;
}

QString qm_ListItem::getAlbum()
{
    return theAlbum;
}

QString qm_ListItem::getArtist()
{
    return theArtist;
}
