#include "qm_ListView.h"
#include "qm_ListItem.h"
#include <qheader.h>
#include <qdragobject.h>

qm_ListView::qm_ListView(QWidget *parent, const char *name)
               : QListView( parent, name )
{
    dropLineItem = 0;
    listType = LV_NONE;	
}


qm_ListView::~qm_ListView()
{ 
    stopLineItem();
}


void qm_ListView::setType(int type)
{
    listType = type;
    if (listType == LV_DROP || listType == LV_DRAGDROP)
	setAcceptDrops(TRUE);
    else
	setAcceptDrops(FALSE);
}


// types: 0 = accepts drops, 1 = allow drags, 2 = drag & drop
void qm_ListView::startDrag()
{
    if(listType == LV_DRAG || listType == LV_DRAGDROP) // accepts drags
    {
        QDragObject *d = new QStoredDrag(QString("qm_DragEvent"), this);
        d->dragCopy();
    }
    // else do nothing
}


void qm_ListView::dragEnterEvent(QDragEnterEvent *event)
{
    if(listType == LV_DROP || listType == LV_DRAGDROP) // accepts drops
	event->accept( event->provides(QString("qm_DragEvent")) ); // from a qm_ListView
    else
	event->accept(0);
}


void qm_ListView::dropEvent(QDropEvent *event)
{
    if(listType == LV_DROP || listType == LV_DRAGDROP) // accepts drops
    {
	stopLineItem();

	if (event->provides(QString("qm_DragEvent"))) // from a qm_ListView
	{
		event->accept(1);
		recieveDrop((qm_ListView *)event->source(), findItem(event->pos()));
	}
	else
		event->accept(0);
    }
    else
        event->accept(0);
}


void qm_ListView::resizeEvent(QResizeEvent *e)
{
    resized(e);
    QListView::resizeEvent(e);
    this->triggerUpdate();
}


void qm_ListView::dragMoveEvent (QDragMoveEvent *e)
{
    qm_ListItem *theItem = findItem(e->pos());
    if(theItem)
    {
        if(theItem == dropLineItem && theItem->getDrawDropLine() == 1 )
		return;

	stopLineItem();

        theItem->setDrawDropLine(TRUE);
        theItem->repaint();
        dropLineItem = theItem;
    }
    else
    {
	theItem = (qm_ListItem *)lastItem();
	if (theItem)
	{
        	theItem->setDrawDropLine(-1); // 0 is FALSE, 1 is TRUE, -1 is 'DRAW at the BOTTOM'
        	theItem->repaint();
        	dropLineItem = theItem;
	}
	else // empty list
		stopLineItem();
     }
}


void qm_ListView::dragLeaveEvent (QDragLeaveEvent *e)
{
    stopLineItem();
}


qm_ListItem *qm_ListView::findItem(QPoint qp)
{
    int verOffset;

    if (this->header()->isHidden())
    	verOffset = 0;
    else
	verOffset = header()->sectionRect(0).height();

    return ((qm_ListItem *)itemAt (QPoint(qp.x(), qp.y() - verOffset)));
}


void qm_ListView::stopLineItem()
{
    if(dropLineItem)
    {
        dropLineItem->setDrawDropLine(FALSE);
        dropLineItem->repaint();
        dropLineItem = 0;
    }
}



