#include <qfile.h>
#include <qprocess.h>
#include "qm_clicklabel.h"
#include "qm_clickprogressbar.h"
#include <qapplication.h>
#include <qmessagebox.h> // error msg etc

QString musicPath;
uchar bR, bG, bB;
bool b_startup;
char kbps_yes; // used to skip kbps updates

// Initialize at startup
void qm_Player::init()
{
    setCaption( "Quimup" );
    setIcon(QPixmap::fromMimeSource( "mn_icon.png" ));
    b_plReallyClose = FALSE;
    b_lockProgress = FALSE;
    b_playerSetsVolume = FALSE;
    b_coreSetsVolume = FALSE;
    b_volDisabled = FALSE;
    b_xfadeWhenDone = FALSE;
    b_timeLeft = FALSE;
    b_sRateFits = TRUE;
    b_isStream = FALSE;
    b_startup = TRUE;
    kbps_yes = 100;
    ScrollPos = 0;
    SongTitleW = 0;
    DisplayW = 320;
    curPlayStatus = -999;
    totalTime = 0;
    curImage = "init"; // anything but the new situation
    // set the tooltip font
    QFont theFont = QApplication::font();
    theFont.setPointSize(theFont.pointSize()-1);
    QToolTip::setFont(theFont);
    QToolTip::setWakeUpDelay(100);
    // volume slider: no valueChanged signal during drag
    sl_volume->setTracking(FALSE);
    sl_volume->setTickmarks(QSlider::NoMarks);
    // set QPixmaps default to best performance
    QPixmap::setDefaultOptimization( QPixmap::BestOptim);
    QSettings settings;
    b_miniMode = settings.readBoolEntry( "/quimup/General/miniMode", FALSE);
    int titleFsize = settings.readNumEntry( "/quimup/General/TitleFontSize", 10);
    plRedrawTitle(titleFsize);
    int infoFsize = settings.readNumEntry( "/quimup/General/InfoFontSize", 9);
    plRedrawInfo(infoFsize);
    int albumFsize = settings.readNumEntry( "/quimup/General/AlbumFontSize", 8);
    plRedrawAlbum(albumFsize);
    // nb. function 'setupTitle' is called from newSong
    // Get the icons
    pauseIcon = QPixmap::fromMimeSource( "mn_pause.png" );
    playIcon  = QPixmap::fromMimeSource( "mn_play.png" );
    statPlay  = QPixmap::fromMimeSource( "led_playing.png" );
    statStop  = QPixmap::fromMimeSource( "led_stopped.png" );
    statNocon = QPixmap::fromMimeSource( "led_noconn.png" );
    statPause = QPixmap::fromMimeSource( "led_paused.png" );

    // to allow easy fading set onair and nopic to fully opaque, using the background color.
    bool colorz = settings.readBoolEntry( "/quimup/General/colorise", TRUE );
    if (colorz)
    {
    bR = 218;
    bG = 220;
    bB = 224;
    }
    else
    {
    QPalette pal (fr_aArt->palette());
    QRgb bg = (pal.color(QPalette::Normal, QColorGroup::Background)).rgb();
    bR = qRed(bg);
    bG = qGreen(bg);
    bB = qBlue(bg);
    }

    // convert 'onair' to image and blend with bg
    prevImage = (QPixmap::fromMimeSource( "onair.png" )).convertToImage();
    for ( int j = 0; j < 200; j++ )
    {
    QRgb *px = (QRgb*)prevImage.scanLine(j);
    for( int k = 0; k < 200; k++ )
    {
        int a = qAlpha(*px);
        uchar r = uchar( (  qRed(*px) * a  +  bR * (255 - a)) / 255 );
        uchar g = uchar( (qGreen(*px) * a  +  bG * (255 - a)) / 255 );
        uchar b = uchar( ( qBlue(*px) * a  +  bB * (255 - a)) / 255 );
        *px = qRgba( r, g, b, 255);
        px++;
    }
    }
    // covert 'onair' back to pixmap
    onAir.convertFromImage(prevImage);

    // fill prevImage with background color
    for ( int j = 0; j < 200; j++ )
    {
    QRgb *px = (QRgb*)prevImage.scanLine(j);
    for( int k = 0; k < 200; k++ )
    {
        *px = qRgba( bR, bG, bB, 255);
        px++;
    }
    }
    thePixmap.convertFromImage(prevImage);

    // init theImage to 'nopic', blended with background
    theImage = (QPixmap::fromMimeSource( "nopic.png" )).convertToImage();
    for ( int j = 0; j < 200; j++ )
    {
    QRgb *px = (QRgb*)theImage.scanLine(j);
    for( int k = 0; k < 200; k++ )
    {
        int a = qAlpha(*px);
        uchar r = uchar( (  qRed(*px) * a  +  bR * (255 - a)) / 255 );
        uchar g = uchar( (qGreen(*px) * a  +  bG * (255 - a)) / 255 );
        uchar b = uchar( ( qBlue(*px) * a  +  bB * (255 - a)) / 255 );
        *px = qRgba( r, g, b, 255);
        px++;
    }
    }
    // covert 'nopic' back to pixmap
    noPic.convertFromImage(theImage);


    // Set icon for the play/pause button
    bt_PlayPause->setPixmap(playIcon);
    // NoBackground results in less flicker (they say).
    lb_mainTitle->setBackgroundMode(NoBackground);
    lb_bitrate->setBackgroundMode(NoBackground);
    lb_nowtime->setBackgroundMode(NoBackground);
    lb_samplerate->setBackgroundMode(NoBackground);
    // Set this window's size fixed (x, y)
    setFixedSize(336, 310);
    // The title scroll-timer
    TitleScroller = new QTimer(this);
    connect( TitleScroller, SIGNAL(timeout()), this, SLOT(scrollStep()) );
    // The mini-maxi slider
    windowSlider = new QTimer(this);
    connect( windowSlider, SIGNAL(timeout()), this, SLOT(slideStep()) );
    // The album-art cross fader
    crossFader = new QTimer(this);
    connect( crossFader, SIGNAL(timeout()), this, SLOT(xfadeStep()) );
    // connect signals from the Core
    connect( parent(), SIGNAL(plRedrawTitle(int)), this, SLOT(plRedrawTitle(int)) );
    connect( parent(), SIGNAL(plRedrawInfo(int)), this, SLOT(plRedrawInfo(int)) );
    connect( parent(), SIGNAL(plRedrawAlbum(int)), this, SLOT(plRedrawAlbum(int)) );
    connect( parent(), SIGNAL(plUpdate(int, int, int)), this, SLOT(plUpdate(int, int, int)) );
    connect( parent(), SIGNAL(plChangeStatus(int)), this, SLOT(plChangeStatus(int)) );
    connect( parent(), SIGNAL(plToggleTimeLeft()), this, SLOT(plToggleTimeLeft()) );
    connect( parent(), SIGNAL(plReallyClose()), this, SLOT(plReallyClose()) );
    connect( parent(), SIGNAL(plNewSong(mpd_Song*, int)), this, SLOT(plNewSong(mpd_Song*, int)) );
    connect( parent(), SIGNAL(plSetRandom(bool)), this, SLOT(plSetRandom(bool)) );
    connect( parent(), SIGNAL(plSetRepeat(bool)), this, SLOT(plSetRepeat(bool)) );
    connect( parent(), SIGNAL(plVolDisabled(bool)), this, SLOT(plVolDisabled(bool)) );
    connect( parent(), SIGNAL(plSetVolume(int)), this, SLOT(plCoreSetVolume(int)) );
    connect( parent(), SIGNAL(plSetErrorMsg(const QString &)), this, SLOT(plSetErrorMsg(const QString &)) );
    // Get the location of the music files
    getMusicPath();
    // get the custom search string for album art
    albumArtString = settings.readEntry( "/quimup/General/albumArt", "" );
    if (albumArtString.isEmpty() || albumArtString.length() < 5)
    b_isArtString = FALSE;
    else
    b_isArtString = TRUE;
    // Set colors for the track-display (background, text)
    plSetDisplayColors(colorz);
    // Finally set the mini/maxi mode
    if (b_miniMode) // don't slide, just set it
    {
    // hide widgets
    sl_volume->hide();
    fr_aArt->hide();
    lb_album->hide();
    rb_repeat->hide();
    rb_random->hide();
    // move buttons up
    bt_prev->move(6, 74);
    bt_stop->move(48, 74);
    bt_PlayPause->move(90, 74);
    bt_next->move(132, 74);
    bt_tracks->move(174, 74);
    px_status->move(223, 80);
    bt_sizer->move(256, 74);
    bt_opts->move(298, 74);
    // set mini window size
    setFixedSize(336, 106);
    }
}


void qm_Player::setMinMaxMode()
{
    if (b_miniMode)     // expand
    {
    alphacycler = 0;
    windowSlider->start(12);
    }
    else            // shrink
    {
    alphacycler = 255;
    windowSlider->start(12);
    }
}

void qm_Player::xfadeStep()
{
    alphacycler -= 10;
    if (alphacycler <= 0) // fading is done
    {
        crossFader->stop();
        fr_aArt->setErasePixmap(thePixmap);
        fr_aArt->setPixmap(thePixmap);
    }
    else
    {
        uchar a = alphacycler;
            theImage = thePixmap.convertToImage();
        for ( int j = 0; j < 200; j++ )
        {
            QRgb *newpx = (QRgb*)theImage.scanLine(j);
            QRgb *prevpx = (QRgb*)prevImage.scanLine(j);
            for( int k = 0; k < 200; k++ )
            {
                uchar r = uchar( (  qRed(*prevpx) * a  +  qRed(*newpx) * (255 - a)) / 255 );
                uchar g = uchar( (qGreen(*prevpx) * a  +  qGreen(*newpx) * (255 - a)) / 255 );
                uchar b = uchar( ( qBlue(*prevpx) * a  +  qBlue(*newpx) * (255 - a)) / 255 );
                *newpx = qRgba( r, g, b, 255);
                newpx++;
                prevpx++;
            }
        }
        QPixmap pMap;
        pMap.convertFromImage(theImage);
        // bitBlt to the WIDGET.
        bitBlt (fr_aArt, 0, 0, &pMap, 0, 0, 200, 200, Qt::CopyROP, FALSE);
    }
}


// Periodically called by the windowSlider timer
void qm_Player::slideStep()
{
    if (b_miniMode) // expanding window
    {
        // 17 steps, from 74 to 278
        int pos =  bt_prev->y() + 12;

        if (pos <= 278) // resizing first, while pos <= 278
        {
            // first expand the window
            setFixedSize(336, pos + 32);

            // and move buttons down
            bt_prev->move(6, pos);
            bt_stop->move(48, pos);
            bt_PlayPause->move(90, pos);
            bt_next->move(132, pos);
            bt_tracks->move(174, pos);
            px_status->move(223, pos + 6 );
            bt_sizer->move(256, pos);
            bt_opts->move(298, pos);

            if (pos == 278)
            {
                // show widgets
                lb_album->show();
                rb_repeat->show();
                rb_random->show();
                sl_volume->show();
            }
        }
        else // resizing is done: fade in the albumart
        {
            // fix the albumart
            alphacycler += 15;
            if (alphacycler >= 255) // All done
            {

                fr_aArt->show();
                b_miniMode = FALSE;
                windowSlider->stop();
                if (b_xfadeWhenDone)
                {
                    b_xfadeWhenDone = FALSE;
                    alphacycler = 255;
                    crossFader->start(12);
                }
                return;
            }
            else
            {
                uchar a = alphacycler;
                theImage = thePixmap.convertToImage();
                    for ( int j = 0; j < 200; j++ )
                    {
                    QRgb *px = (QRgb*)theImage.scanLine(j);
                    for( int k = 0; k < 200; k++ )
                    {
                        uchar r = uchar( (  qRed(*px) * a  +  bR * (255 - a)) / 255 );
                        uchar g = uchar( (qGreen(*px) * a  +  bG * (255 - a)) / 255 );
                        uchar b = uchar( ( qBlue(*px) * a  +  bB * (255 - a)) / 255 );
                        *px = qRgba( r, g, b, 255);
                        px++;
                    }
                    }
                QPixmap pMap;
                pMap.convertFromImage(theImage);
                // bitBlt to the WINDOW. Original fr_aArt is shown on top when all is done (above)
                bitBlt (this, 6, 72, &pMap, 0, 0, 200, 200, Qt::CopyROP, FALSE);
            }
        }
    }
    else // collapsing window
    {
        if ( alphacycler > 0) // fade out the album art
        {
            alphacycler -= 15;
            if (alphacycler <= 0) // fading is done
            {
                // hide widgets
                fr_aArt->hide();
                sl_volume->hide();
                lb_album->hide();
                rb_repeat->hide();
                rb_random->hide();
            }
            else
            {
                uchar a = alphacycler;
                theImage = thePixmap.convertToImage();
                for ( int j = 0; j < 200; j++ )
                {
                    QRgb *px = (QRgb*)theImage.scanLine(j);
                    for( int k = 0; k < 200; k++ )
                    {
                        uchar r = uchar( (  qRed(*px) * a  +  bR * (255 - a)) / 255 );
                        uchar g = uchar( (qGreen(*px) * a  +  bG * (255 - a)) / 255 );
                        uchar b = uchar( ( qBlue(*px) * a  +  bB * (255 - a)) / 255 );
                        *px = qRgba( r, g, b, 255);
                        px++;
                    }
                }
                QPixmap pMap;
                pMap.convertFromImage(theImage);
                // bitBlt to the WIDGET. fr_aArt is hidden when fading is done (above),
                // and will display thePixmap when fr_aArt->show() again
                bitBlt (fr_aArt, 0, 0, &pMap, 0, 0, 200, 200, Qt::CopyROP, FALSE);
            }

        }
        else // fading is done: resize now
        {
            // 17 steps, from 278 to 74
            int pos =  bt_prev->y() - 12;

            //  move buttons up
            bt_prev->move(6, pos);
            bt_stop->move(48, pos);
            bt_PlayPause->move(90, pos);
            bt_next->move(132, pos);
            bt_tracks->move(174, pos);
            px_status->move(223, pos + 6 );
            bt_sizer->move(256, pos);
            bt_opts->move(298, pos);

            // next shrink the window
            setFixedSize(336, pos + 32);

            if (pos == 74) // All done.
            {
                b_miniMode = TRUE;
                windowSlider->stop();
                if (b_xfadeWhenDone)
                {
                    b_xfadeWhenDone = FALSE;
                    alphacycler = 255;
                    crossFader->start(12);
                }
                return;
            }
        }
    }
}


void qm_Player::plChangeStatus(int playStatus)
{
    // nothing to do
    if (curPlayStatus == playStatus)
    return;

    if (playStatus == -99) // a call from plSetErrorMsg (always happens at start-up)
    {
    ds_progress->setProgress(0);
        // Blank the labels
        // lb_mainTitle is set trough 'setupTitle'
    lb_bitrate->setText("");
    lb_kbps->setText("");
    lb_nowtime->setText("");
    lb_extension->setText("");
    lb_samplerate->setText("");
        lb_kHz->setText("");
        lb_totaltime->setText("");
    // In case we come from 'paused' set the p/p button to play
    bt_PlayPause->setPixmap(playIcon);
    TitleScroller->stop();
    sl_volume->setEnabled(FALSE);
    bt_prev->setEnabled(FALSE);
    bt_stop->setEnabled(FALSE);
    bt_PlayPause->setEnabled(FALSE);
    bt_next->setEnabled(FALSE);
    ds_progress->setEnabled(FALSE);
    rb_random->setEnabled(FALSE);
    rb_repeat->setEnabled(FALSE);
    px_status->setPixmap(statNocon);
    // skip at startup: this always happens before mpd is connected.
    if (!b_startup)
            plNewSong(NULL, -1);
    }
    else
    {
    // re-enable buttons after an error
        if (curPlayStatus == -99)
    {
        if(!b_volDisabled && playStatus)
            sl_volume->setEnabled(TRUE);
        bt_prev->setEnabled(TRUE);
        bt_stop->setEnabled(TRUE);
        bt_PlayPause->setEnabled(TRUE);
        bt_next->setEnabled(TRUE);
        ds_progress->setEnabled(TRUE);
        rb_random->setEnabled(TRUE);
        rb_repeat->setEnabled(TRUE);
    }
    }

    // Adjust labels etc for current status
    if(playStatus == MPD_STATUS_STATE_STOP || playStatus == -99)
    {
    ds_progress->setProgress(0);
        // Blank the labels
    lb_bitrate->setText("");
    lb_kbps->setText("");
    lb_nowtime->setText("");
    lb_extension->setText("");
    lb_samplerate->setText("");
        lb_kHz->setText("");
        lb_totaltime->setText("");
    // In case we come from 'paused' set the p/p button to play
    bt_PlayPause->setPixmap(playIcon);
    TitleScroller->stop();
    setupTitle("Stopped");
        crSetTrayTip("Stopped");
    px_status->setPixmap(statStop);
    }
    else
    if(playStatus == MPD_STATUS_STATE_PLAY)
    {
    bt_PlayPause->setPixmap(pauseIcon);
    px_status->setPixmap(statPlay);
    lb_kbps->setText("kbps");
    if(b_sRateFits)
        lb_kHz->setText("kHz");
    }
    else
    if (playStatus == MPD_STATUS_STATE_PAUSE)
    {
    bt_PlayPause->setPixmap(playIcon);
    px_status->setPixmap(statPause);
    lb_kbps->setText("kbps");
    if(b_sRateFits)
        lb_kHz->setText("kHz");
    }

    // Save status
    curPlayStatus = playStatus;
}


// Called from init() and from the Core (when fonts are changed in Options)
void qm_Player::plRedrawInfo(int infoFsize)
{
    QFont theFont;
    theFont.setFamily("MonoSpace");
    theFont.setBold(FALSE);
    if (infoFsize < 11)
    theFont.setPointSize(infoFsize - 1);
    else
    theFont.setPointSize(infoFsize - 2);

    lb_kbps->setFont(theFont);
    lb_kHz->setFont(theFont);


    theFont.setPointSize(infoFsize);

    lb_samplerate->setFont(theFont);
    lb_bitrate->setFont(theFont);
    lb_extension->setFont(theFont);
    lb_totaltime->setFont(theFont);

    theFont.setBold(TRUE);

    if (infoFsize < 11)
    theFont.setPointSize(infoFsize + 1);
    else
    theFont.setPointSize(infoFsize + 2);

    lb_nowtime->setFont(theFont);
    // Set label positions, based on the actual font:
    // (main title is positioned in 'setupTitle')
    int pos = 10;
    int textW = QFontMetrics(lb_extension->font()).width("WAVE");
    lb_extension->setGeometry(pos, 27, textW, 18); // file type

    pos += (textW + 10);
    textW = QFontMetrics(lb_bitrate->font()).width("0000");
    lb_bitrate->setGeometry(pos , 27, textW, 18); // kbps #

    pos += (textW + 4);
    textW = QFontMetrics(lb_kbps->font()).width("kbps");
    lb_kbps->setGeometry(pos, 28, textW, 18); // "kbps" label

    int sRatePos = pos;
    pos += (textW + 12);
    textW = QFontMetrics(lb_samplerate->font()).width("44.4");
    lb_samplerate->setGeometry(pos , 27, textW, 18); // kHz #

    pos += (textW + 4);
    textW = QFontMetrics(lb_kHz->font()).width("kHz");
    lb_kHz->setGeometry(pos, 28, textW, 18); // "kHz" label

    // time info is positioned R to L
    int RtextW = QFontMetrics(lb_totaltime->font()).width(" / 00:00");
    lb_totaltime->setGeometry(DisplayW-(RtextW + 3), 27, RtextW, 18); // total time

    int Rpos = RtextW;
    RtextW = QFontMetrics(lb_nowtime->font()).width("-00:00");
    lb_nowtime->setGeometry(DisplayW-(RtextW + 3 + Rpos), 27, RtextW, 18); // play time

    // Large Info Font: no space for 'samplerate' & "kHz"
    if ((pos + textW + 4) > DisplayW-(RtextW + 3 + Rpos))
    {
    b_sRateFits = FALSE;
    lb_kHz->setText("");
    lb_kHz->setGeometry(sRatePos, 0, 0, 0); // "kHz" label
    lb_samplerate->setText("");
        lb_samplerate->setGeometry(sRatePos , 0, 0, 0); // kHz #
    }
    else
    {
    b_sRateFits = TRUE;
    if (curPlayStatus == MPD_STATUS_STATE_PLAY || curPlayStatus == MPD_STATUS_STATE_PAUSE)
        lb_kHz->setText("kHz");
    }
}

void qm_Player::plRedrawTitle(int titleFsize)
{
    QFont theFont = QApplication::font();
    theFont.setBold(TRUE);
    theFont.setPointSize(titleFsize);
    lb_mainTitle->setFont(theFont);
    // check if it still fits // scrolling on/off?
}

void qm_Player::plRedrawAlbum(int albumFsize)
{
    QFont theFont = QApplication::font();
    theFont.setBold(FALSE);
    theFont.setPointSize(albumFsize);
    lb_album->setFont(theFont);
}


// Position the main title
void qm_Player::setupTitle(const QString & title)
{
    ScrollPos = 0;
    lb_mainTitle->setText(title);
    // reposition the text
    lb_mainTitle->setGeometry(6,1,QFontMetrics(lb_mainTitle->font()).width(lb_mainTitle->text()), 16);
}


// Periodically called by the TitleScroller
void qm_Player::scrollStep()
{
    //Get the current size of the title bar
    QRect titleRect = lb_mainTitle->geometry();

    // reset when scrolled through all of the text
    if (ScrollPos > SongTitleW)
    {
    titleRect.setLeft(titleRect.left() + ScrollPos);
    titleRect.setWidth(titleRect.width() + ScrollPos);
    ScrollPos = 0;
    }

    //move the title
    ScrollPos += 1;
    titleRect.setLeft(titleRect.left() - 1);
    titleRect.setWidth(titleRect.width() - 1);
    lb_mainTitle->setGeometry(titleRect);
}

// valueChanged on volume slider
void qm_Player::setVolume()
{
    if(!b_coreSetsVolume)
    {
    b_playerSetsVolume = TRUE;
    crSendVolume(sl_volume->value());
    b_playerSetsVolume = FALSE;
    }
}

// Click down on progress slider
void qm_Player::LockProgress(QMouseEvent *e)
{
    if(curPlayStatus == MPD_STATUS_STATE_PLAY)
    {
    b_lockProgress = TRUE;
    // Get mouse x pos within widget
    int x = e->x();
    x = (x*1000) / 324; // 324 is width of widget, 1000 is # of steps
    ds_progress->setProgress(x);
    }

}

// Click up on progress slider. Signal to Core.
void qm_Player::unLockProgress()
{
    if(curPlayStatus == MPD_STATUS_STATE_PLAY)
    {
    crSendProgress((double)(ds_progress->progress()) / (double)1000.0);
    b_lockProgress = FALSE;
    }
}

// Toggle + and - time
void qm_Player::plToggleTimeLeft()
{
    if (!b_isStream)
    {
    b_timeLeft = !b_timeLeft;
    // send to Core so it can be saved later
    crStoreTimeLeft(b_timeLeft);
    }
}

// Signal to Core
void qm_Player::tRepeat()
{
    crToggleRepeat();
}

// Signal to Core
void qm_Player::tRandom()
{
    crToggleRandom();
}

// Signal to Core
void qm_Player::tTracks()
{
    crToggleTracks();
}

// Signal to Core
void qm_Player::sOptions()
{
    crToggleOptions();
}

// Signal to Core
void qm_Player::sNext()
{
    crSendNext();
}

// Signal to Core
void qm_Player::sPrev()
{
    crSendPrev();
}

// Signal to Core
void qm_Player::sPlay()
{
    crSendPlay();
}

// Signal to Core
void qm_Player::sStop()
{
    crSendStop();
}

// Signal from the Core. Periodically updates the time / birate / samplerate / progress.
void qm_Player::plUpdate(int nowTime, int songBitrate, int songSamplerate )
{
    QString formattedTime = "";

    // Set the time
    if(nowTime < 0)
    nowTime = 0;

    int time;
    if(b_timeLeft && totalTime != 0) // streams have totalTime 0
    {
    time = totalTime - nowTime;
    if(time < 0 )
        time = 0;
    }
    else
    time = nowTime;

    // format the time
    int mins = (int)(time / 60);
    formattedTime += QString::number(mins) + ":";
    int secs = time % 60;
    if(secs < 10)
    formattedTime += "0";
    formattedTime += QString::number(secs);

    if(b_timeLeft && totalTime != 0) // streams have totalTime 0
    {
    formattedTime = "-" + formattedTime;
    }

    lb_nowtime->setText(formattedTime);

    // Update bitrate and samplerate (may not be constant)
    if (kbps_yes >= 6)
    {
    lb_bitrate->setText(QString::number(songBitrate));

    double sRate = (double)songSamplerate/1000.0;
     if(b_sRateFits)
        lb_samplerate->setText(QString::number(sRate)); // QString converts to 1 decimal point

    kbps_yes = 0; // reset
    }
    kbps_yes ++;

    // Set the progress bar.
    if(totalTime > 0)
    {
    int currentPos = (int)(1000 * ((double)nowTime / (double)totalTime));
    if(!b_lockProgress) ds_progress->setProgress(currentPos);
    }
}

// Signal from the Core. Sets up a new song.
void qm_Player::plNewSong( mpd_Song * aSong, int songNr )
{

    if (aSong == NULL || songNr < 0) // no song!
    {
    if (curImage == "nopIc")
    return;

    // we start up with a blanc prevImage (see init)
        if (!b_startup)
        prevImage = thePixmap.convertToImage();
    else
        b_startup = FALSE;

    thePixmap = noPic;
    curImage = "nopIc";

    lb_album->setText("");
        QToolTip::remove (fr_aArt );

    if ( windowSlider->isActive() )// wait for slider to finish
        b_xfadeWhenDone = TRUE;
    else
    if (b_miniMode) // don't bother to xfade
    {
        fr_aArt->setErasePixmap(thePixmap);
        fr_aArt->setPixmap(thePixmap);
    }
    else
    {
        alphacycler = 255;
        crossFader->start(12);
    }
    return;
    }

    QString aString;
    QString theTrayTip;
    // the filename (this first: If the path is a URL this is a stream)
    QString fileName = QString::fromUtf8(aSong->file).stripWhiteSpace();
    if (fileName.startsWith("http:"))
    b_isStream = TRUE;
    else
    b_isStream = FALSE;

    int i = fileName.findRev( '/' ); // remove path
    fileName = fileName.right(fileName.length() - ++i);

    // the song play-time
    totalTime = aSong->time;
    if (totalTime <= 0 || b_isStream)
    {
    totalTime = 0; // disables timeLeft
    lb_totaltime->setText("");
    }
    else
    {
    if(curPlayStatus != MPD_STATUS_STATE_STOP)
    {
        QString formattedTime = " / ";
        int mins = (int)(totalTime / 60);
        formattedTime += QString::number(mins) + ":";
        int secs = totalTime % 60;
        if(secs < 10)
            formattedTime += "0";
        formattedTime += QString::number(secs);
        lb_totaltime->setText(formattedTime);
    }
    }

    // the comment
    QToolTip::remove (fr_aArt );
    bool b_isComment = FALSE;
    Comment = QString::fromUtf8(aSong->comment);
    if (!Comment.isEmpty())
    {
    b_isComment = TRUE;
    QToolTip::add (fr_aArt, Comment );
    }

    // the file extension
    if(curPlayStatus != MPD_STATUS_STATE_STOP)
    {
    if (b_isStream)
    {
        lb_extension->setText("URL");
    }
    else
    {
        aString = fileName;
        i = aString.findRev( '.' );
        aString = aString.right(aString.length() - ++i);
        aString = aString.upper();   // string to upper case
        lb_extension->setText(aString);
    }
    }

    // Setup main title string & tray-icon tooltip, depending on available info
    if(curPlayStatus != MPD_STATUS_STATE_STOP)
    {
    QString theMainTitle = QString::fromUtf8(aSong->title).stripWhiteSpace();
    if (theMainTitle.isEmpty())
    {
        theMainTitle = fileName;
        theTrayTip = "<nobr>&nbsp;File: <b>" + fileName + "</b>&nbsp;</nobr>";
    }
    else
    {
        theTrayTip = "<nobr>&nbsp;" + theMainTitle + "&nbsp;</nobr>";
        QString arTist = QString::fromUtf8(aSong->artist).stripWhiteSpace();
            if (!arTist.isEmpty())
        {
                theTrayTip = "<nobr><b>&nbsp;" + arTist + "</b></nobr><br>" + theTrayTip;
            theMainTitle = arTist + " : " + theMainTitle;
        }
        else
        {
            if (b_isStream)
                theTrayTip = "&nbsp;<b>Audio stream</b><br>" + theTrayTip;
            else
            {
                    theTrayTip = "&nbsp;<b>[ ??? ]</b><br>" + theTrayTip;
                theMainTitle = "[ ??? ] : " + theMainTitle;
            }
        }
        }

    //If the size of the song title is BIGGER than (Display_Width - (2 x Xoffset))
    if (QFontMetrics(lb_mainTitle->font()).width(theMainTitle) > DisplayW - 12)
    {
        aString = theMainTitle + " ::: ";
        SongTitleW = QFontMetrics(lb_mainTitle->font()).width(aString);
            if(curPlayStatus != MPD_STATUS_STATE_STOP)
        {
            setupTitle(theMainTitle+" ::: " + theMainTitle);
            TitleScroller->start(100);
        }
    }
    else
    {
        setupTitle(theMainTitle);
        TitleScroller->stop();
        }
    }


    // Setup the album info
    QString albumLabel = "";
    if (b_isStream)
    {
    albumLabel = "<b>Audio Stream</b>";
    }
    else
    {
    aString = QString::fromUtf8(aSong->album).stripWhiteSpace();
    if (aString.isEmpty())
        aString = "<b> ??? </b></> ";
    else
        aString = "<b>" + aString + " </b>";
    // Note that a space is added above.
    // without it single-word album names are not displayed correctly. (QT rich-text bug?)
    albumLabel.append(aString);

    aString = QString::fromUtf8(aSong->track);
    if (!aString.isEmpty())
    {
            // add leading zero if 1 digit
        if (aString.length() == 1)
            aString = "0" + aString;
        albumLabel.append(" #" + aString);
    }


    aString = QString::fromUtf8(aSong->date);
    if (!aString.isEmpty())
        albumLabel.append("<hr>" + aString);

    aString = QString::fromUtf8(aSong->genre);
    if (!aString.isEmpty())
        albumLabel.append("<hr>" + aString);
    else
    {

            if (b_isComment)
            albumLabel.append("<br>&lsaquo; comment");
    }

        if (b_isComment)
        albumLabel.append("<br>&lsaquo; comment");
    }

    lb_album->setText(albumLabel);

    aString = QString::fromUtf8(aSong->title).stripWhiteSpace();
    if (aString.isEmpty()) aString = "?";

    aString = QString::fromUtf8(aSong->track);
    if (aString.isEmpty()) aString = "?";
    else
    {
        // add leading zero if 1 digit
        if (aString.length() == 1)
        aString = "0" + aString;
    }

    // Set the tray-icon tooltip
    crSetTrayTip(theTrayTip);

    // Set up the Album Art & Comment
    bool imageChange = FALSE;
    bool b_found = FALSE;

    // First time we use a blanc prevImage (set in init)
    if (!b_startup)
    prevImage = thePixmap.convertToImage();
    else
    b_startup = FALSE;

    if (b_isStream)
    {
    if (curImage != "onAir")
    {
        thePixmap = onAir;
        curImage = "onAir";
        imageChange = TRUE;
    }
    }
    else
    {
    if (musicPath != "error")
    {
        aString = QString::fromUtf8(aSong->file);
        i = aString.findRev( '/' );
        aString = aString.left(i);     // mpd's relative path
        aString = musicPath + aString; // full path to art dir

        QDir artdir(aString);
        QStringList allFiles = artdir.entryList(QDir::Files);
        QString matchingFile = "not found";

        // first scheck for the custom album-art string
        if (b_isArtString)
        for ( QStringList::Iterator it = allFiles.begin(); it != allFiles.end(); ++it)
        {
            QString str = *it;
            if ( str.contains(albumArtString, FALSE) &&
                ( str.endsWith(".jpg", FALSE) || str.endsWith(".png", FALSE) ||
                str.endsWith(".gif", FALSE) || str.endsWith(".bmp", FALSE) ) )
            {
                matchingFile = str;
                aString.append("/" + matchingFile);
                b_found = TRUE;
                break;
            }
        }

        if (!b_found) // Check for "folder."
        for ( QStringList::Iterator it = allFiles.begin(); it != allFiles.end(); ++it)
        {
            QString str = *it;
            if ( str.contains("folder.", FALSE) &&
                ( str.endsWith(".jpg", FALSE) || str.endsWith(".png", FALSE) ||
                str.endsWith(".gif", FALSE) || str.endsWith(".bmp", FALSE) ) )
            {
                matchingFile = str;
                aString.append("/" + matchingFile);
                b_found = TRUE;
                break;
            }
        }

        if (!b_found) // Check for "album"
        for ( QStringList::Iterator it = allFiles.begin(); it != allFiles.end(); ++it)
        {
            QString str = *it;
            if (str.contains("album", FALSE) &&
                ( str.endsWith(".jpg", FALSE) || str.endsWith(".png", FALSE) ||
                str.endsWith(".gif", FALSE) || str.endsWith(".bmp", FALSE) ) )
            {
                matchingFile = str;
                aString.append("/" + matchingFile);
                b_found = TRUE;
                break;
            }
        }

        if (!b_found) // Check for "front"
        for ( QStringList::Iterator it = allFiles.begin(); it != allFiles.end(); ++it)
        {
            QString str = *it;
            if (str.contains("front", FALSE) &&
                ( str.endsWith(".jpg", FALSE) || str.endsWith(".png", FALSE) ||
                str.endsWith(".gif", FALSE) || str.endsWith(".bmp", FALSE) ) )
            {
                matchingFile = str;
                aString.append("/" + matchingFile);
                b_found = TRUE;
                break;
            }
        }

        if (!b_found) // Check for 'album name'
        for ( QStringList::Iterator it = allFiles.begin(); it != allFiles.end(); ++it)
        {
            QString str = *it;
            if (str.contains(QString::fromUtf8(aSong->album), FALSE) &&
                ( str.endsWith(".jpg", FALSE) || str.endsWith(".png", FALSE) ||
                str.endsWith(".gif", FALSE) || str.endsWith(".bmp", FALSE) ) )
            {
                matchingFile = str;
                aString.append("/" + matchingFile);
                b_found = TRUE;
                break;
            }
        }


        if (curImage != aString)
        {
             // if we have a valid path and image can be loaded
            if (b_found && theImage.load(aString, 0))
            {
                curImage = aString;
                // resize the image
                theImage  = theImage.smoothScale ( 200, 200, QImage::ScaleFree );
                // remove transparency and add border
                    for ( int i = 0; i < 200; i++ )
                    {
                    QRgb *px = (QRgb*)theImage.scanLine(i);
                    // if top or bottom row: border
                    if (i == 0 || i == 199)
                    {
                        for ( int j = 0; j < 200; j++)
                        {
                            *px = qRgb( 43, 63, 73 );
                            px++;
                        }
                    }
                    else
                    for( int k = 0; k < 200; k++ )
                    {
                        if (k == 0 || k == 199) //left or right column: border
                            *px = qRgb( 43, 63, 73 );
                        else
                        {
                            int a = qAlpha(*px);
                            uchar r = uchar( (  qRed(*px) * a  +  bR * (255 - a)) / 255 );
                            uchar g = uchar( (qGreen(*px) * a  +  bG * (255 - a)) / 255 );
                            uchar b = uchar( ( qBlue(*px) * a  +  bB * (255 - a)) / 255 );
                            *px = qRgba( r, g, b, 255);
                        }
                        px++;
                    }
                    }
                // load new image into a pixmap
                if ( !(thePixmap.convertFromImage(theImage)) )
                {
                    curImage = "nopIc";
                    thePixmap = noPic;
                }
                imageChange = TRUE;
            }
            else
            {
                if (curImage != "nopIc")
                {

                    curImage = "nopIc";
                    thePixmap = noPic;
                    imageChange = TRUE;
                }
            }
        }
    } // else musicPath equals "error"
    } // end 'not a stream'


    if(imageChange)
    {
    if ( windowSlider->isActive() )// wait for slider to finish
        b_xfadeWhenDone = TRUE;
    else
    if (b_miniMode) // don't bother to xfade
    {
        fr_aArt->setErasePixmap(thePixmap);
        fr_aArt->setPixmap(thePixmap);
    }
    else
    {
        alphacycler = 255;
        crossFader->start(12);
    }
    }

}


void qm_Player::getMusicPath()
{
    QFile conf;
    QSettings settings;
    // get the mpd config file.
    QString cfgPath = settings.readEntry( "/quimup/Server/configPath", "x" );
    if (cfgPath == "x" || cfgPath == "unknown location" || !(QFile::exists (cfgPath)))
    musicPath = "error";
    else
    conf.setName(cfgPath);

    if ( musicPath != "error" && conf.open( IO_ReadOnly ) )
    {
        QTextStream stream( &conf );
        QString line;
        int i;
        while (!stream.atEnd() )
        {
            line = stream.readLine().stripWhiteSpace();
            if (line.startsWith("music_dir", FALSE))
            {
                i = line.find('"') + 1;
                line = line.right(line.length() - i);
                i = line.find('"');
                line.truncate(i);
                if (line[line.length() - 1] != '/')
                    line = line + "/";
                musicPath = line;
            }

        }
        conf.close();
    }
    else
        musicPath = "error";

    // Check if the music dir really is accessible
        QDir musDir(musicPath);
    if (!musDir.isReadable())
    {
            musicPath = "error";
        QMessageBox::information( this, "Quimup",
            "Could not open music directory.\nDo you have access rights?");
    }
}


// Signal from the Core.
void qm_Player::plSetRandom( bool value )
{
    rb_random->setChecked(value);
}

// Signal from the Core.
void qm_Player::plSetRepeat( bool value )
{
    rb_repeat->setChecked(value);
}

// Signal from the Core.
void qm_Player::plVolDisabled( bool value )
{
    b_volDisabled = value;
    sl_volume->setEnabled(!value);
}

// Signal from the Core.
void qm_Player::plCoreSetVolume( int value )
{
    if(!b_playerSetsVolume)
    {
    b_coreSetsVolume = TRUE;
    sl_volume->setValue(value);
    b_coreSetsVolume = FALSE;
    }
}

// Called when this window is closed
void qm_Player::closeEvent( QCloseEvent * a )
{
    if(!b_plReallyClose)
    {
    a->ignore();
        // hide by faking a 'trayclicked'
    crTrayClicked(-99);
    crPlayerClosed();
    }
    else
    a->accept();
}

// Signal from the Core.
void qm_Player::plReallyClose()
{
    QSettings settings;
    settings.writeEntry( "/quimup/General/miniMode", b_miniMode);
    // This time allow closing,
    b_plReallyClose = TRUE;
    // and trigger closeEvent.
    this->close();
}



void qm_Player::plSetDisplayColors(bool colorize)
{
    QColor fg_col (30, 60, 90);
    QColor bg_col (211, 228, 240);

    // colors for other widgets
    if (colorize)
    {
    QColor fCol = QColor(0, 10, 40);
    QPalette pal (sl_volume->palette()); // just grab any palette
    pal.setColor(QColorGroup::ButtonText, fCol);
    pal.setColor(QColorGroup::Text, fCol);
    pal.setColor(QColorGroup::Foreground, fCol);
    pal.setColor(QColorGroup::Background, QColor(218, 220, 224));
    pal.setColor(QColorGroup::Button, QColor(240, 245, 250));
    pal.setColor(QColorGroup::Highlight, QColor(75, 100, 125));
    pal.setColor(QColorGroup::Shadow, QColor(15, 30, 50));
    pal.setColor(QColorGroup::Mid, QColor(116, 130, 148));
    // set palette globally
    setPalette(pal);
    // make progressbar lighter than 'highlight'
    pal.setColor(QColorGroup::Highlight, QColor(114, 138, 163));
    ds_progress->setPalette(pal);
    // Artist label gets same text color as main display
    lb_album->setPaletteForegroundColor(fg_col);
    // radio-buttons 'on' color
    rb_random->setPaletteForegroundColor(QColor(60, 110, 135));
    rb_repeat->setPaletteForegroundColor(QColor(60, 110, 135));
    }

    // always set colors for the main display
    Display->setPaletteBackgroundColor(bg_col);
    Display->setPaletteForegroundColor(fg_col);
    lb_bitrate->setPaletteBackgroundColor(bg_col);
    lb_bitrate->setPaletteForegroundColor(fg_col);
    lb_kbps->setPaletteBackgroundColor(bg_col);
    lb_kbps->setPaletteForegroundColor(fg_col);
    lb_nowtime->setPaletteBackgroundColor(bg_col);
    lb_nowtime->setPaletteForegroundColor(fg_col);
    lb_mainTitle->setPaletteBackgroundColor(bg_col);
    lb_mainTitle->setPaletteForegroundColor(fg_col);
    lb_samplerate->setPaletteBackgroundColor(bg_col);
    lb_samplerate->setPaletteForegroundColor(fg_col);
    lb_totaltime->setPaletteBackgroundColor(bg_col);
    lb_totaltime->setPaletteForegroundColor(fg_col);
    lb_kHz->setPaletteBackgroundColor(bg_col);
    lb_kHz->setPaletteForegroundColor(fg_col);
    lb_extension->setPaletteBackgroundColor(bg_col);
    lb_extension->setPaletteForegroundColor(fg_col);
}

// Signal from the Core.
void qm_Player::plSetErrorMsg(const QString & Message )
{
    plChangeStatus(-99);
    setupTitle(Message);
            crSetTrayTip("Error");
}

