#ifndef QOPS_H
#define QOPS_H

#include <qsettings.h>
#include <qapplication.h>

void qm_Settings::init()
{
    // caption icon
    setIcon(QPixmap::fromMimeSource( "mn_config.png" ));

    // palette
    QSettings settings;
    if (settings.readBoolEntry( "/quimup/General/colorise", TRUE ))
    {
    QColor fCol = QColor(0, 10, 40);
    QPalette pal (tabWidget->palette()); // grab any palette
    pal.setColor(QColorGroup::ButtonText, fCol);
    pal.setColor(QColorGroup::Text, fCol);
    pal.setColor(QColorGroup::Foreground, fCol);
    pal.setColor(QColorGroup::Background, QColor(218, 220, 224));
    pal.setColor(QColorGroup::Button, QColor(240, 245, 250));
    pal.setColor(QColorGroup::Highlight, QColor(75, 100, 125));
    pal.setColor(QColorGroup::Shadow, QColor(15, 30, 50));
    pal.setColor(QColorGroup::Mid, QColor(116, 130, 148));

    setPalette(pal);
    // no 'button-color' on the scrollbar in the about textbox
    pal.setColor(QColorGroup::Button, QColor(218, 220, 224));
    tb_about->setPalette(pal);
    }

    // set fonts
    QFont smallFont = QApplication::font();
    smallFont.setPointSize(smallFont.pointSize()-1);

    QFont miniFont = QApplication::font();
    miniFont.setPointSize(miniFont.pointSize()-2);

    QFont bigFont = QApplication::font();
    bigFont.setPointSize(bigFont.pointSize()+1);
    bigFont.setBold(TRUE);

    lb_metadata->setFont(smallFont);
    lb_mpdchar->setFont(smallFont);
    lb_conffile->setFont(smallFont);
    lb_replaygain->setFont(smallFont);
    lb_user->setFont(smallFont);
    lb_mixerControl->setFont(smallFont);
    lb_audioutput->setFont(smallFont);
    lb_musicDir->setFont(smallFont);
    lb_playlistDir->setFont(smallFont);
    lb_makesure->setText("Make sure mpd is actually using this configuration file!");

    lb_makesure->setFont(miniFont);
    // 'about' icon and title
    px_about->setPixmap(QPixmap::fromMimeSource( "mn_icon.png" ));

    tl_about->setFont(bigFont);
    // version nr goes here:
    tl_about->setText("Quimup 0.3.5");

    // connections
    connect( parent(), SIGNAL(stsGetsMPData(const mpdOptStruct &)), this, SLOT(stsGetsMPData(const mpdOptStruct &)) );
    connect( parent(), SIGNAL(stsSetConMsg(bool, const QString &)), this, SLOT(stsSetConMsg(bool, const QString &)) );
    connect( parent(), SIGNAL(stsGetsOptions(const qm_OptStruct &)), this, SLOT(stsGetsOptions(const qm_OptStruct &)) );
    connect( parent(), SIGNAL(stsSetVersion(const QString &)), this, SLOT(stsSetVersion(const QString &)) );
}

// Load saved options
void qm_Settings::stsGetsOptions(const qm_OptStruct & opts)
{
    cb_useSysTray->setChecked(opts.useSystray);
    cb_colorise->setChecked(opts.colorise);
    cb_startServer->setChecked(opts.startServer);
    cb_autoConn->setChecked(opts.autoConnect);
    cb_killServer->setChecked(opts.killServer);
    le_killcommand->setText(opts.killCmd);
    le_startcommand->setText(opts.startCmd);
    le_hostName->setText(opts.sName);
    le_portText->setText(QString::number(opts.sPort));
    if (opts.sPassword != "")
        le_passText->setText("***");
    else
        le_passText->setText("");
    le_listFont->setText(opts.listFont);
    le_albumArt->setText(opts.albumArt);
    sb_crossFade->setValue(opts.xfadeTime);
    sb_titleFontsz->setValue(opts.titleFsize);
    sb_infoFontsz->setValue(opts.infoFsize);
    sb_albumFontsz->setValue(opts.albumFsize);
    sb_listFontsz->setValue(opts.listFsize);
    // Set up the output devices
    OutDevList::iterator cit;
    mpd_OutputEntity curDev;
    OutDevList theList = opts.oDevices;
    // dev1
    cit = theList.begin();
    if (cit < theList.end())
    {
    cb_dev1->setEnabled(TRUE);
    curDev = *cit;
    cb_dev1->setText(QString::fromUtf8(curDev.name));
        if (curDev.enabled)
        cb_dev1->setChecked(TRUE);
    else
        cb_dev1->setChecked(FALSE);
    }
    else
    {
    cb_dev1->setText("Is mpd connected?");
    cb_dev1->setChecked(FALSE);
    cb_dev1->setEnabled(FALSE);
    }
    // dev2
    ++cit;
    if (cit < theList.end())
    {
    cb_dev2->setEnabled(TRUE);
    curDev = *cit;
    cb_dev2->setText(QString::fromUtf8(curDev.name));
        if (curDev.enabled)
        cb_dev2->setChecked(TRUE);
    else
        cb_dev2->setChecked(FALSE);
    }
    else
    {
    cb_dev2->setText("n.a.");
    cb_dev2->setChecked(FALSE);
    cb_dev2->setEnabled(FALSE);
    }
    ++cit;
    // dev3
    if (cit < theList.end())
    {
    cb_dev3->setEnabled(TRUE);
    curDev = *cit;
    cb_dev3->setText(QString::fromUtf8(curDev.name));
        if (curDev.enabled)
        cb_dev3->setChecked(TRUE);
    else
        cb_dev3->setChecked(FALSE);
    }
    else
    {
    cb_dev3->setText("n.a.");
    cb_dev3->setChecked(FALSE);
    cb_dev3->setEnabled(FALSE);
    }
    ++cit;
    // dev4
    if (cit < theList.end())
    {
    cb_dev4->setEnabled(FALSE);
    curDev = *cit;
    cb_dev4->setText(QString::fromUtf8(curDev.name));
        if (curDev.enabled)
        cb_dev4->setChecked(TRUE);
    else
        cb_dev4->setChecked(FALSE);
    }
    else
    {
    cb_dev4->setText("n.a.");
    cb_dev4->setChecked(FALSE);
    cb_dev4->setEnabled(FALSE);
    }
}

// Set strings in MPD config tab
void qm_Settings::stsGetsMPData(const mpdOptStruct & mpdOptions)
{
    bool infok = mpdOptions.infoFound;
    if (infok)
    {
        le_conffile->setText(mpdOptions.configPath);
    lb_metadata->setText("Meta data used:  " + mpdOptions.metaData);
        lb_metadata->setAlignment(WordBreak|AlignLeft|AlignVCenter);
    lb_mpdchar->setText("Character set:  " + mpdOptions.chars);
    lb_conffile->setText("/path/file");
    lb_replaygain->setText("Replaygain mode:  " + mpdOptions.replayGain);
    lb_user->setText("User:  " + mpdOptions.user);
    lb_mixerControl->setText("Mixer control:  " + mpdOptions.mixerControl);
    lb_audioutput->setText("Outputs:  " + mpdOptions.audiOutput);
    lb_musicDir->setText("Music dir:  " + mpdOptions.musicPath);
    lb_playlistDir->setText("Playlist dir:  " + mpdOptions.playlistPath);
    }
    else
    {
    le_conffile->setText("enter location here");
    lb_metadata->setText("Meta data used:  ?");
    lb_mpdchar->setText("Character set:  ?");
    lb_conffile->setText("/path/file");
    lb_replaygain->setText("Replaygain mode:  ?");
    lb_user->setText("User:  ?");
    lb_mixerControl->setText("Mixer control:  ?");
    lb_audioutput->setText("Outputs:  ?");
    lb_musicDir->setText("Music dir:  ?");
    lb_playlistDir->setText("Playlist dir:  ?");
    }

    lb_metadata->setEnabled(infok);
    lb_mpdchar->setEnabled(infok);
    lb_conffile->setEnabled(infok);
    lb_replaygain->setEnabled(infok);
    lb_user->setEnabled(infok);
    lb_mixerControl->setEnabled(infok);
    lb_audioutput->setEnabled(infok);
    lb_musicDir->setEnabled(infok);
    lb_playlistDir->setEnabled(infok);
}


void qm_Settings::ResetAllFonts()
{
    sb_titleFontsz->setValue(10);
    sb_infoFontsz->setValue(9);
    sb_listFontsz->setValue(10);
    sb_albumFontsz->setValue(9);
    le_listFont->setText("SansSerif");
}


void qm_Settings::cancelChanges()
{
    this->close();
}


void qm_Settings::saveChanges()
{
    qm_OptStruct newOps;
    newOps.useSystray = cb_useSysTray->isChecked();
    newOps.colorise = cb_colorise->isChecked();
    newOps.xfadeTime = sb_crossFade->value();
    newOps.startServer = cb_startServer->isChecked();
    newOps.startCmd = (le_startcommand->text()).stripWhiteSpace();
    newOps.killServer = cb_killServer->isChecked();
    newOps.killCmd = (le_killcommand->text()).stripWhiteSpace();
    newOps.autoConnect = cb_autoConn->isChecked();
    newOps.sName = (le_hostName->text()).stripWhiteSpace();
    newOps.configPath = (le_conffile->text()).stripWhiteSpace();
    newOps.sPassword = (le_passText->text()).stripWhiteSpace();
    newOps.sPort = le_portText->text().toInt();
    newOps.listFont = le_listFont->text().stripWhiteSpace();
    newOps.titleFsize = sb_titleFontsz->value();
    newOps.infoFsize = sb_infoFontsz->value();
    newOps.albumFsize = sb_albumFontsz->value();
    newOps.listFsize = sb_listFontsz->value();
    QString str = (le_albumArt->text()).stripWhiteSpace();
    if (str.length() < 5)
    newOps.albumArt = "";
    else
    newOps.albumArt = str;

    // output devices:
    mpd_OutputEntity addDev;
    // dev1
    if (cb_dev1->isEnabled())
    {
    addDev.id = 0;
    addDev.name = (char*)"dev0";
    addDev.enabled = cb_dev1->isChecked();
    (newOps.oDevices).append(addDev);
    }
    // dev2
    if (cb_dev2->isEnabled())
    {
    addDev.id = 1;
    addDev.name = (char*)"dev1";
    addDev.enabled = cb_dev2->isChecked();
    (newOps.oDevices).append(addDev);
    }
    // dev3
    if (cb_dev3->isEnabled())
    {
    addDev.id = 2;
    addDev.name = (char*)"dev2";
    addDev.enabled = cb_dev3->isChecked();
    (newOps.oDevices).append(addDev);
    }
    // dev4
    if (cb_dev4->isEnabled())
    {
    addDev.id = 3;
    addDev.name = (char*)"dev3";
    addDev.enabled = cb_dev4->isChecked();
    (newOps.oDevices).append(addDev);
    }

    crGetsOptions(newOps);

    (newOps.oDevices).clear();
    this->close();
}


void qm_Settings::serverConnect()
{
    QString passWd = noTrail(le_passText->text()).ascii();
    QString hostNm = noTrail(le_hostName->text());
    int portNr = le_portText->text().toInt();
    crServerConnect(hostNm, portNr, passWd);
}

// Set the connection status string
void qm_Settings::stsSetConMsg(bool connected, const QString & errorMsg)
{
    if (connected)
    {
    lb_statusText->setText("Status: Connected!");
    lb_connIcon->setPixmap(QPixmap::fromMimeSource( "con_ok.png" ));
    }
    else
    {
    lb_statusText->setText(errorMsg);
    lb_connIcon->setPixmap(QPixmap::fromMimeSource( "con_dis.png" ));
    }

}

// Set mpd version string
void qm_Settings::stsSetVersion(const QString & version)
{
    lb_mpdVersion->setText(version);
}


void qm_Settings::closeEvent( QCloseEvent * a)
{
    //Accept the close event
    a->accept();
}

// Strips the trailing spaces off typed strings
QString qm_Settings::noTrail(const QString & sIn)
{
    QString sOut = sIn;

    while (sOut.length() > 0 && sOut[sOut.length() - 1].isSpace())
    sOut.truncate(sOut.length()-1);

    return sOut;
}

#endif
